@@ -70,7 +70,7 @@ VLC_BOOTSTRAP_ARGS="\
     --disable-dca \
     --disable-goom \
     --disable-chromaprint \
-    --disable-lua \
+    --enable-lua \
     --disable-schroedinger \
     --disable-sdl \
     --disable-SDL_image \
@@ -91,11 +91,13 @@ VLC_BOOTSTRAP_ARGS="\
     --enable-iconv \
     --disable-aribb24 \
     --disable-aribb25 \
-    --disable-mpg123 \
+    --enable-mpg123 \
     --enable-libdsm \
     --enable-libarchive \
     --disable-libmpeg2 \
     --enable-soxr \
+    --enable-nfs \
+    --enable-microdns \
 "
 
 ###########################
@@ -117,7 +119,7 @@ VLC_CONFIGURE_ARGS="\
     --disable-update-check \
     --disable-vlm \
     --disable-dbus \
-    --disable-lua \
+    --enable-lua \
     --disable-vcd \
     --disable-v4l2 \
     --disable-gnomevfs \
@@ -219,7 +221,6 @@ VLC_MODULE_BLACKLIST="
     fb
     aout_file
     yuv
-    mpg123
     .dummy
 "
 
@@ -618,19 +619,23 @@ EOF
     BUILTINS="$BUILTINS vlc_entry__$name,\n";
 done;
 BUILTINS="$BUILTINS NULL\n};\n"; \
-printf "/* Autogenerated from the list of modules */\n$DEFINITION\n$BUILTINS\n" > libvlc/jni/libvlcjni-modules.h
+printf "/* Autogenerated from the list of modules */\n#include <unistd.h>\n$DEFINITION\n$BUILTINS\n" > libvlc/jni/libvlcjni-modules.c
+
+DEFINITION=""
+BUILTINS="const void *libvlc_functions[] = {\n";
+for func in `cat vlc/lib/libvlc.sym`
+do
+    DEFINITION=$DEFINITION"int $func(void);\n";
+    BUILTINS="$BUILTINS $func,\n";
+done
+BUILTINS="$BUILTINS NULL\n};\n"; \
+printf "/* Autogenerated from the list of modules */\n#include <unistd.h>\n$DEFINITION\n$BUILTINS\n" > libvlc/jni/libvlcjni-symbols.c
+
 rm ${REDEFINED_VLC_MODULES_DIR}/syms
 
-# Generating the .ver file like libvlc.so upstream
-VER_FILE="vlc/$VLC_BUILD_DIR/lib/.libs/libvlc.ver"
-echo "{ global:" > $VER_FILE
-cat vlc/lib/libvlc.sym libvlc/libvlcjni.sym | sed -e "s/\(.*\)/\1;/" >> $VER_FILE
-echo "__gmp_binvert_limb_table;" >> $VER_FILE # FIXME
-echo "local: *; };" >> $VER_FILE
-
-###############################
-# NDK-Build for libvlcjni.so  #
-###############################
+############################################
+# NDK-Build for libvlc.so and libvlcjni.so #
+############################################
 
 LIBVLC_LIBS="libvlcjni"
 VLC_MODULES=$(find_modules ${REDEFINED_VLC_MODULES_DIR})
@@ -660,7 +665,7 @@ $ANDROID_NDK/ndk-build -C libvlc \
     VLC_CONTRIB="$VLC_CONTRIB" \
     VLC_MODULES="$VLC_MODULES" \
     TARGET_CFLAGS="$EXTRA_CFLAGS" \
-    EXTRA_LDFLAGS="$EXTRA_LDFLAGS -Wl,-soname -Wl,libvlc.so.5 -Wl,-version-script -Wl,$SRC_DIR/$VER_FILE" \
+    EXTRA_LDFLAGS="$EXTRA_LDFLAGS" \
     LIBVLC_LIBS="$LIBVLC_LIBS" \
     LIBIOMX_LIBS="$LIBIOMX_LIBS" \
     LIBANW_LIBS="$LIBANW_LIBS" \
@@ -676,18 +681,28 @@ $ANDROID_NDK/ndk-build -C libvlc \
 
 checkfail "ndk-build failed"
 
-if [ "$RELEASE" = 1 ]; then
-    DBG_LIB_DIR=libvlc/jni/obj/local/${ANDROID_ABI}
-    OUT_LIB_DIR=libvlc/jni/libs/${ANDROID_ABI}
-    VERSION=$(grep "android:versionName" vlc-android/AndroidManifest.xml|cut -d\" -f 2)
-    OUT_DBG_DIR=.dbg/${ANDROID_ABI}/$VERSION
-
-    mkdir -p $OUT_DBG_DIR
-
-    for lib in ${DBG_LIB_DIR}/*.so; do
-        ${CROSS_COMPILE}objcopy --only-keep-debug "$lib" "$OUT_DBG_DIR/`basename $lib.dbg`"; \
-    done
-    for lib in ${OUT_LIB_DIR}/*.so; do
-        ${CROSS_COMPILE}objcopy --add-gnu-debuglink="$OUT_DBG_DIR/`basename $lib.dbg`" "$lib" ; \
-    done
+if [ "${ANDROID_API}" = "android-9" ] && [ "${ANDROID_ABI}" = "armeabi-v7a" -o "${ANDROID_ABI}" = "armeabi" ] ; then
+    $ANDROID_NDK/ndk-build -C libvlc \
+        APP_BUILD_SCRIPT=libcompat/Android.mk \
+        APP_PLATFORM=${ANDROID_API} \
+        APP_ABI="armeabi" \
+        NDK_PROJECT_PATH=libcompat \
+        NDK_TOOLCHAIN_VERSION=${GCCVER} \
+        NDK_DEBUG=${NDK_DEBUG}
+    checkfail "ndk-build compat failed"
 fi
+
+DBG_LIB_DIR=libvlc/jni/obj/local/${ANDROID_ABI}
+OUT_LIB_DIR=libvlc/jni/libs/${ANDROID_ABI}
+VERSION=$(grep "android:versionName" vlc-android/AndroidManifest.xml|cut -d\" -f 2)
+OUT_DBG_DIR=.dbg/${ANDROID_ABI}/$VERSION
+
+echo "Dumping dbg symbols info ${OUT_DBG_DIR}"
+
+mkdir -p $OUT_DBG_DIR
+for lib in ${DBG_LIB_DIR}/*.so; do
+    ${CROSS_COMPILE}objcopy --only-keep-debug "$lib" "$OUT_DBG_DIR/`basename $lib.dbg`"; \
+done
+for lib in ${OUT_LIB_DIR}/*.so; do
+    ${CROSS_COMPILE}objcopy --add-gnu-debuglink="$OUT_DBG_DIR/`basename $lib.dbg`" "$lib" ; \
+done
