@@ -48,6 +48,7 @@ import android.preference.PreferenceManager;
 import android.support.annotation.MainThread;
 import android.support.annotation.Nullable;
 import android.support.v4.app.NotificationManagerCompat;
+import android.support.v4.content.LocalBroadcastManager;
 import android.support.v4.media.MediaMetadataCompat;
 import android.support.v4.media.session.MediaSessionCompat;
 import android.support.v4.media.session.PlaybackStateCompat;
@@ -66,11 +67,14 @@ import org.videolan.libvlc.MediaPlayer;
 import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.vlc.gui.AudioPlayerContainerActivity;
 import org.videolan.vlc.gui.helpers.AudioUtil;
+import org.videolan.vlc.gui.preferences.PreferencesActivity;
+import org.videolan.vlc.gui.preferences.PreferencesFragment;
 import org.videolan.vlc.gui.video.VideoPlayerActivity;
 import org.videolan.vlc.media.MediaDatabase;
 import org.videolan.vlc.media.MediaUtils;
 import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.media.MediaWrapperList;
+import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.Strings;
 import org.videolan.vlc.util.Util;
@@ -105,7 +109,8 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     public static final String ACTION_REMOTE_STOP = ACTION_REMOTE_GENERIC+"Stop";
     public static final String ACTION_REMOTE_FORWARD = ACTION_REMOTE_GENERIC+"Forward";
     public static final String ACTION_REMOTE_LAST_PLAYLIST = ACTION_REMOTE_GENERIC+"LastPlaylist";
-    public static final String ACTION_REMOTE_RESUME_VIDEO = ACTION_REMOTE_GENERIC+"ResumeVideo";
+    public static final String ACTION_REMOTE_LAST_VIDEO_PLAYLIST = ACTION_REMOTE_GENERIC+"LastVideoPlaylist";
+    public static final String ACTION_REMOTE_SWITCH_VIDEO = ACTION_REMOTE_GENERIC+"SwitchToVideo";
 
     public interface Callback {
         void update();
@@ -124,6 +129,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         return binder.getService();
     }
 
+    private SharedPreferences mSettings;
     private final IBinder mBinder = new LocalBinder();
     private MediaWrapperList mMediaList = new MediaWrapperList();
     private MediaPlayer mMediaPlayer;
@@ -132,6 +138,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     private boolean mPausable = false;
     private boolean mIsAudioTrack = false;
     private boolean mHasHdmiAudio = false;
+    private boolean mSwitchingToVideo = false;
 
     final private ArrayList<Callback> mCallbacks = new ArrayList<Callback>();
     private boolean mDetectHeadset = true;
@@ -158,13 +165,16 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             | PlaybackStateCompat.ACTION_PLAY | PlaybackStateCompat.ACTION_STOP
             | PlaybackStateCompat.ACTION_SKIP_TO_NEXT | PlaybackStateCompat.ACTION_SKIP_TO_PREVIOUS;
 
+    public static final int TYPE_AUDIO = 0;
+    public static final int TYPE_VIDEO = 1;
+
     public static final int REPEAT_NONE = 0;
     public static final int REPEAT_ONE = 1;
     public static final int REPEAT_ALL = 2;
     private boolean mShuffling = false;
     private int mRepeating = REPEAT_NONE;
     private Random mRandom = null; // Used in shuffling process
-
+    private long mSavedTime = 0l;
     private boolean mHasAudioFocus = false;
     // RemoteControlClient-related
     /**
@@ -182,9 +192,8 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     }
 
     private MediaPlayer newMediaPlayer() {
-        final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
         final MediaPlayer mp = new MediaPlayer(LibVLC());
-        final String aout = VLCOptions.getAout(pref);
+        final String aout = VLCOptions.getAout(mSettings);
         if (mp.setAudioOutput(aout) && aout.equals("android_audiotrack")) {
             mIsAudioTrack = true;
             if (mHasHdmiAudio)
@@ -204,15 +213,19 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     public void onCreate() {
         super.onCreate();
 
+        mSettings = PreferenceManager.getDefaultSharedPreferences(this);
         mMediaPlayer = newMediaPlayer();
+        mMediaPlayer.setEqualizer(VLCOptions.getEqualizer(this));
 
         if (!VLCInstance.testCompatibleCPU(this)) {
             stopSelf();
             return;
         }
 
-        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
-        mDetectHeadset = prefs.getBoolean("enable_headset_detection", true);
+        if (!AndroidDevices.hasTsp() && !AndroidDevices.hasPlayServices())
+            AndroidDevices.setRemoteControlReceiverEnabled(true);
+
+        mDetectHeadset = mSettings.getBoolean("enable_headset_detection", true);
 
         mCurrentIndex = -1;
         mPrevIndex = -1;
@@ -235,7 +248,8 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         filter.addAction(ACTION_REMOTE_STOP);
         filter.addAction(ACTION_REMOTE_FORWARD);
         filter.addAction(ACTION_REMOTE_LAST_PLAYLIST);
-        filter.addAction(ACTION_REMOTE_RESUME_VIDEO);
+        filter.addAction(ACTION_REMOTE_LAST_VIDEO_PLAYLIST);
+        filter.addAction(ACTION_REMOTE_SWITCH_VIDEO);
         filter.addAction(VLCAppWidgetProvider.ACTION_WIDGET_INIT);
         filter.addAction(Intent.ACTION_HEADSET_PLUG);
         filter.addAction(AudioManager.ACTION_AUDIO_BECOMING_NOISY);
@@ -243,8 +257,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         registerReceiver(mReceiver, filter);
         registerV21();
 
-        final SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
-        boolean stealRemoteControl = pref.getBoolean("enable_steal_remote_control", false);
+        boolean stealRemoteControl = mSettings.getBoolean("enable_steal_remote_control", false);
 
         if (!AndroidUtil.isFroyoOrLater() || stealRemoteControl) {
             /* Backward compatibility for API 7 */
@@ -277,12 +290,12 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             if (hasCurrentMedia())
                 return START_STICKY;
             else
-                loadLastPlaylist();
+                loadLastPlaylist(TYPE_AUDIO);
         } else if (ACTION_REMOTE_PLAY.equals(intent.getAction())) {
             if (hasCurrentMedia())
                 play();
             else
-                loadLastPlaylist();
+                loadLastPlaylist(TYPE_AUDIO);
         }
         updateWidget();
         return super.onStartCommand(intent, flags, startId);
@@ -292,6 +305,10 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     public void onDestroy() {
         super.onDestroy();
         stop();
+
+        if (!AndroidDevices.hasTsp() && !AndroidDevices.hasPlayServices())
+            AndroidDevices.setRemoteControlReceiverEnabled(false);
+
         if (mWakeLock.isHeld())
             mWakeLock.release();
         unregisterReceiver(mReceiver);
@@ -316,7 +333,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
     @Override
     public boolean onUnbind(Intent intent) {
-        if (!isPlaying())
+        if (!hasCurrentMedia())
             stopSelf();
         return true;
     }
@@ -342,9 +359,9 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                 switch (focusChange) {
                     case AudioManager.AUDIOFOCUS_LOSS:
                         Log.i(TAG, "AUDIOFOCUS_LOSS");
-                        // Stop playback
+                        // Pause playback
                         changeAudioFocus(false);
-                        stop();
+                        pause();
                         break;
                     case AudioManager.AUDIOFOCUS_LOSS_TRANSIENT:
                         Log.i(TAG, "AUDIOFOCUS_LOSS_TRANSIENT");
@@ -380,7 +397,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     }
 
     @TargetApi(Build.VERSION_CODES.FROYO)
-    private void changeAudioFocusFroyoOrLater(boolean acquire) {
+    private void changeAudioFocus(boolean acquire) {
         final AudioManager am = (AudioManager)getSystemService(AUDIO_SERVICE);
         if (am == null)
             return;
@@ -403,11 +420,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         }
     }
 
-    private void changeAudioFocus(boolean acquire) {
-        if (AndroidUtil.isFroyoOrLater())
-            changeAudioFocusFroyoOrLater(acquire);
-    }
-
 
     @TargetApi(Build.VERSION_CODES.LOLLIPOP)
     private void registerV21() {
@@ -474,8 +486,11 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             } else if (action.equalsIgnoreCase(ACTION_REMOTE_FORWARD)) {
                 next();
             } else if (action.equalsIgnoreCase(ACTION_REMOTE_LAST_PLAYLIST)) {
-                loadLastPlaylist();
-            } else if (action.equalsIgnoreCase(ACTION_REMOTE_RESUME_VIDEO)) {
+                loadLastPlaylist(TYPE_AUDIO);
+            } else if (action.equalsIgnoreCase(ACTION_REMOTE_LAST_VIDEO_PLAYLIST)) {
+                loadLastPlaylist(TYPE_VIDEO);
+            } else if (action.equalsIgnoreCase(ACTION_REMOTE_SWITCH_VIDEO)) {
+                getCurrentMediaWrapper().removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
                 switchToVideo();
             } else if (action.equalsIgnoreCase(VLCAppWidgetProvider.ACTION_WIDGET_INIT)) {
                 updateWidget();
@@ -513,12 +528,16 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
     @Override
     public void onSurfacesCreated(IVLCVout vlcVout) {
-        handleVout();
+        hideNotification(false);
     }
 
     @Override
     public void onSurfacesDestroyed(IVLCVout vlcVout) {
-        handleVout();
+        mSwitchingToVideo = false;
+    }
+
+    @Override
+    public void onHardwareAccelerationError(IVLCVout vlcVout) {
     }
 
     private final Media.EventListener mMediaListener = new Media.EventListener() {
@@ -527,16 +546,13 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             switch (event.type) {
                 case Media.Event.MetaChanged:
                     /* Update Meta if file is already parsed */
-                    if (!mParsed)
-                        break;
+                    if (mParsed && updateCurrentMeta(event.getMetaId()))
+                        executeUpdate();
                     Log.i(TAG, "Media.Event.MetaChanged: " + event.getMetaId());
-
+                    break;
                 case Media.Event.ParsedChanged:
                     Log.i(TAG, "Media.Event.ParsedChanged");
-                    final MediaWrapper mw = getCurrentMedia();
-                    if (mw != null)
-                        mw.updateMeta(mMediaPlayer);
-                    executeUpdate();
+                    updateCurrentMeta(-1);
                     mParsed = true;
                     break;
 
@@ -546,7 +562,23 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         }
     };
 
+    /**
+     * Update current media meta and return true if player needs to be updated
+     *
+     * @param id of the Meta event received, -1 for none
+     * @return true if UI needs to be updated
+     */
+    private boolean updateCurrentMeta(int id) {
+        if (id == Media.Meta.Publisher)
+            return false;
+        final MediaWrapper mw = getCurrentMedia();
+        if (mw != null)
+            mw.updateMeta(mMediaPlayer);
+        return id != Media.Meta.NowPlaying || getCurrentMedia().getNowPlaying() != null;
+    }
+
     private final MediaPlayer.EventListener mMediaPlayerListener = new MediaPlayer.EventListener() {
+
         @Override
         public void onEvent(MediaPlayer.Event event) {
             switch (event.type) {
@@ -576,16 +608,18 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                     }
 
                     changeAudioFocus(true);
-                    showNotification();
                     if (!mWakeLock.isHeld())
                         mWakeLock.acquire();
+                    if (switchToVideo())
+                        hideNotification();
+                    else
+                        showNotification();
                     break;
                 case MediaPlayer.Event.Paused:
                     Log.i(TAG, "MediaPlayer.Event.Paused");
                     executeUpdate();
                     publishState(event.type);
                     executeUpdateProgress();
-                    showNotification();
                     if (mWakeLock.isHeld())
                         mWakeLock.release();
                     break;
@@ -600,7 +634,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                     break;
                 case MediaPlayer.Event.EndReached:
                     Log.i(TAG, "MediaPlayer.Event.EndReached");
-                    executeUpdate();
                     executeUpdateProgress();
                     determinePrevAndNextIndices(true);
                     next();
@@ -626,12 +659,9 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                 case MediaPlayer.Event.Vout:
                     break;
                 case MediaPlayer.Event.ESAdded:
-                    if (event.getEsChangedType() == Media.Track.Type.Video) {
-                        if (!handleVout()) {
-                            /* Update notification content intent: resume video or resume audio activity */
-                            updateMetadata();
-                            showNotification();
-                        }
+                    if (event.getEsChangedType() == Media.Track.Type.Video && !switchToVideo()) {
+                        /* Update notification content intent: resume video or resume audio activity */
+                        updateMetadata();
                     }
                     break;
                 case MediaPlayer.Event.ESDeleted:
@@ -708,22 +738,21 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         return hasCurrentMedia() && mMediaPlayer.getVideoTracksCount() > 0;
     }
 
-    private boolean handleVout() {
-        if (!canSwitchToVideo() || !mMediaPlayer.isPlaying())
-            return false;
-        if (mMediaPlayer.getVLCVout().areViewsAttached()) {
-            hideNotification(false);
-            return true;
-        } else
-            return false;
-    }
-
     @MainThread
     public boolean switchToVideo() {
-        if (!canSwitchToVideo())
+        if (mMediaList.getMedia(mCurrentIndex).hasFlag(MediaWrapper.MEDIA_FORCE_AUDIO) || !canSwitchToVideo())
             return false;
-        if (!mMediaPlayer.getVLCVout().areViewsAttached())
-            VideoPlayerActivity.startOpened(VLCApplication.getAppContext(), mCurrentIndex);
+        if (isVideoPlaying()) {//Player is already running, just send it an intent
+            mMediaPlayer.setVideoTrackEnabled(true);
+            LocalBroadcastManager.getInstance(this).sendBroadcast(
+                    VideoPlayerActivity.getIntent(VideoPlayerActivity.PLAY_FROM_SERVICE,
+                            getCurrentMediaWrapper(), false, mCurrentIndex));
+        } else if (!mSwitchingToVideo) {//Start the video player
+            Log.e(TAG, "startOpened", new Exception());
+            VideoPlayerActivity.startOpened(VLCApplication.getAppContext(),
+                    getCurrentMediaWrapper().getUri(), mCurrentIndex);
+            mSwitchingToVideo = true;
+        }
         return true;
     }
 
@@ -738,7 +767,6 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         if (updateWidget)
             updateWidget();
         updateMetadata();
-
     }
 
     private void executeUpdateProgress() {
@@ -801,11 +829,14 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         if (mMediaPlayer.getVLCVout().areViewsAttached())
             return;
         try {
+            boolean coverOnLockscreen = mSettings.getBoolean("lockscreen_cover", true);
             MediaMetadataCompat metaData = mMediaSession.getController().getMetadata();
             String title = metaData.getString(MediaMetadataCompat.METADATA_KEY_TITLE);
             String artist = metaData.getString(MediaMetadataCompat.METADATA_KEY_ALBUM_ARTIST);
             String album = metaData.getString(MediaMetadataCompat.METADATA_KEY_ALBUM);
-            Bitmap cover = metaData.getBitmap(MediaMetadataCompat.METADATA_KEY_ALBUM_ART);
+            Bitmap cover = coverOnLockscreen ?
+                    metaData.getBitmap(MediaMetadataCompat.METADATA_KEY_ALBUM_ART) :
+                    AudioUtil.getCover(this, getCurrentMedia(), 512);
             if (cover == null)
                 cover = BitmapFactory.decodeResource(VLCApplication.getAppContext().getResources(), R.drawable.icon);
             Notification notification;
@@ -828,9 +859,9 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
 
             PendingIntent pendingIntent;
-            if (canSwitchToVideo()) {
-                /* Resume VideoPlayerActivity from from ACTION_REMOTE_RESUME_VIDEO intent */
-                final Intent notificationIntent = new Intent(ACTION_REMOTE_RESUME_VIDEO);
+            if (canSwitchToVideo() && !mMediaList.getMedia(mCurrentIndex).hasFlag(MediaWrapper.MEDIA_FORCE_AUDIO) ) {
+                /* Resume VideoPlayerActivity from ACTION_REMOTE_SWITCH_VIDEO intent */
+                final Intent notificationIntent = new Intent(ACTION_REMOTE_SWITCH_VIDEO);
                 pendingIntent = PendingIntent.getBroadcast(this, 0, notificationIntent, PendingIntent.FLAG_UPDATE_CURRENT);
             } else {
                 /* Resume AudioPlayerActivity */
@@ -1054,6 +1085,21 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         public void onSkipToPrevious() {
             previous();
         }
+
+        @Override
+        public void onSeekTo(long pos) {
+            setTime(pos);
+        }
+
+        @Override
+        public void onFastForward() {
+            next();
+        }
+
+        @Override
+        public void onRewind() {
+            previous();
+        }
     }
 
     protected void updateMetadata() {
@@ -1065,7 +1111,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         String title = media.getNowPlaying();
         if (title == null)
             title = media.getTitle();
-        Bitmap cover = AudioUtil.getCover(this, media, 512);
+        boolean coverOnLockscreen = mSettings.getBoolean("lockscreen_cover", true);
         MediaMetadataCompat.Builder bob = new MediaMetadataCompat.Builder();
         bob.putString(MediaMetadataCompat.METADATA_KEY_TITLE, title)
             .putString(MediaMetadataCompat.METADATA_KEY_GENRE, MediaUtils.getMediaGenre(this, media))
@@ -1073,8 +1119,11 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
                 .putString(MediaMetadataCompat.METADATA_KEY_ALBUM_ARTIST, MediaUtils.getMediaReferenceArtist(this, media))
                 .putString(MediaMetadataCompat.METADATA_KEY_ALBUM, MediaUtils.getMediaAlbum(this, media))
                 .putLong(MediaMetadataCompat.METADATA_KEY_DURATION, media.getLength());
-        if (cover != null && cover.getConfig() != null) //In case of format not supported
+        if (coverOnLockscreen) {
+            Bitmap cover = AudioUtil.getCover(this, media, 512);
+            if (cover != null && cover.getConfig() != null) //In case of format not supported
                 bob.putBitmap(MediaMetadataCompat.METADATA_KEY_ALBUM_ART, cover.copy(cover.getConfig(), false));
+        }
         mMediaSession.setMetadata(bob.build());
 
         //Send metadata to Pebble watch
@@ -1085,6 +1134,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             i.putExtra("track", media.getTitle());
             sendBroadcast(i);
         }
+        showNotification();
     }
 
     protected void publishState(int state) {
@@ -1138,12 +1188,13 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             if (mCurrentIndex < 0)
                 saveCurrentMedia();
             Log.w(TAG, "Warning: invalid next index, aborted !");
+            //Close video player if started
+            LocalBroadcastManager.getInstance(this).sendBroadcast(new Intent(VideoPlayerActivity.EXIT_PLAYER));
             stop();
             return;
         }
-
         playIndex(mCurrentIndex, 0);
-        onMediaChanged();
+        saveCurrentMedia();
     }
 
     @MainThread
@@ -1162,7 +1213,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
             setPosition(0f);
 
         playIndex(mCurrentIndex, 0);
-        onMediaChanged();
+        saveCurrentMedia();
     }
 
     @MainThread
@@ -1246,57 +1297,87 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         sendBroadcast(broadcast);
     }
 
-    private synchronized void loadLastPlaylist() {
-        SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
-        String currentMedia = prefs.getString("current_media", "");
+    public synchronized void loadLastPlaylist(int type) {
+        boolean audio = type == TYPE_AUDIO;
+        String currentMedia = mSettings.getString(audio ? "current_song" : "current_media", "");
         if (currentMedia.equals(""))
             return;
-        String[] locations = prefs.getString("media_list", "").split(" ");
+        String[] locations = mSettings.getString(audio ? "audio_list" : "media_list", "").split(" ");
+        if (locations.length == 0)
+            return;
 
         List<String> mediaPathList = new ArrayList<String>(locations.length);
         for (int i = 0 ; i < locations.length ; ++i)
             mediaPathList.add(Uri.decode(locations[i]));
 
-        mShuffling = prefs.getBoolean("shuffling", false);
-        mRepeating = prefs.getInt("repeating", REPEAT_NONE);
-        int position = prefs.getInt("position_in_list", Math.max(0, mediaPathList.indexOf(currentMedia)));
-        long time = prefs.getLong("position_in_song", -1);
+        mShuffling = mSettings.getBoolean(audio ? "audio_shuffling" : "media_shuffling", false);
+        mRepeating = mSettings.getInt(audio ? "audio_repeating" : "media_repeating", REPEAT_NONE);
+        int position = mSettings.getInt(audio ? "position_in_audio_list" : "position_in_media_list",
+                Math.max(0, mediaPathList.indexOf(currentMedia)));
+        long time = mSettings.getLong(audio ? "position_in_song" : "position_in_media", -1);
+        mSavedTime = time;
         // load playlist
         loadLocations(mediaPathList, position);
         if (time > 0)
             setTime(time);
-        SharedPreferences.Editor editor = prefs.edit();
-        editor.putInt("position_in_list", 0);
-        editor.putLong("position_in_song", 0);
+        if(!audio) {
+            boolean paused = mSettings.getBoolean(PreferencesActivity.VIDEO_PAUSED, !isPlaying());
+            float rate = mSettings.getFloat(PreferencesActivity.VIDEO_SPEED, getRate());
+            if (paused)
+                pause();
+            if (rate != 1.0f)
+                setRate(rate);
+        }
+        SharedPreferences.Editor editor = mSettings.edit();
+        editor.putInt(audio ? "position_in_audio_list" : "position_in_media_list", 0);
+        editor.putLong(audio ? "position_in_song" : "position_in_media", 0);
         Util.commitPreferences(editor);
     }
 
     private synchronized void saveCurrentMedia() {
-        SharedPreferences.Editor editor = PreferenceManager.getDefaultSharedPreferences(this).edit();
-        editor.putString("current_media", mMediaList.getMRL(Math.max(mCurrentIndex, 0)));
+        boolean audio = true;
+        for (int i = 0; i < mMediaList.size(); i++) {
+            if (mMediaList.getMedia(i).getType() == MediaWrapper.TYPE_VIDEO)
+                audio = false;
+        }
+        SharedPreferences.Editor editor = mSettings.edit();
+        editor.putString(audio ? "current_song" : "current_media", mMediaList.getMRL(Math.max(mCurrentIndex, 0)));
         Util.commitPreferences(editor);
     }
 
     private synchronized void saveMediaList() {
-        if (getCurrentMedia() == null || getCurrentMedia().getType() == MediaWrapper.TYPE_VIDEO)
+        if (getCurrentMedia() == null)
             return;
         StringBuilder locations = new StringBuilder();
-        for (int i = 0; i < mMediaList.size(); i++)
+        boolean audio = true;
+        for (int i = 0; i < mMediaList.size(); i++) {
+            if (mMediaList.getMedia(i).getType() == MediaWrapper.TYPE_VIDEO)
+                audio = false;
             locations.append(" ").append(Uri.encode(mMediaList.getMRL(i)));
+        }
         //We save a concatenated String because putStringSet is APIv11.
-        SharedPreferences.Editor editor = PreferenceManager.getDefaultSharedPreferences(this).edit();
-        editor.putString("media_list", locations.toString().trim());
+        SharedPreferences.Editor editor = mSettings.edit();
+        editor.putString(audio ? "audio_list" : "media_list", locations.toString().trim());
         Util.commitPreferences(editor);
     }
 
     private synchronized void savePosition(){
-        if (getCurrentMedia() == null || getCurrentMedia().getType() == MediaWrapper.TYPE_VIDEO)
+        if (getCurrentMedia() == null)
             return;
-        SharedPreferences.Editor editor = PreferenceManager.getDefaultSharedPreferences(this).edit();
-        editor.putBoolean("shuffling", mShuffling);
-        editor.putInt("repeating", mRepeating);
-        editor.putInt("position_in_list", mCurrentIndex);
-        editor.putLong("position_in_song", mMediaPlayer.getTime());
+        SharedPreferences.Editor editor = mSettings.edit();
+        boolean audio = true;
+        for (int i = 0; i < mMediaList.size(); i++) {
+            if (mMediaList.getMedia(i).getType() == MediaWrapper.TYPE_VIDEO)
+                audio = false;
+        }
+        editor.putBoolean(audio ? "audio_shuffling" : "media_shuffling", mShuffling);
+        editor.putInt(audio ? "audio_repeating" : "media_repeating", mRepeating);
+        editor.putInt(audio ? "position_in_audio_list" : "position_in_media_list", mCurrentIndex);
+        editor.putLong(audio ? "position_in_song" : "position_in_media", mMediaPlayer.getTime());
+        if(!audio) {
+            editor.putBoolean(PreferencesActivity.VIDEO_PAUSED, !isPlaying());
+            editor.putFloat(PreferencesActivity.VIDEO_SPEED, getRate());
+        }
         Util.commitPreferences(editor);
     }
 
@@ -1378,8 +1459,8 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     public String getArtist() {
         if (hasCurrentMedia()) {
             final MediaWrapper media = getCurrentMedia();
-            return media.isArtistUnknown() && media.getNowPlaying() != null ?
-                    media.getNowPlaying()
+            return media.getNowPlaying() != null ?
+                    media.getTitle()
                     : MediaUtils.getMediaArtist(PlaybackService.this, media);
         } else
             return null;
@@ -1404,7 +1485,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     @MainThread
     public String getTitle() {
         if (hasCurrentMedia())
-            return getCurrentMedia().getTitle();
+            return getCurrentMedia().getNowPlaying() != null ? getCurrentMedia().getNowPlaying() : getCurrentMedia().getTitle();
         else
             return null;
     }
@@ -1504,7 +1585,7 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
 
     @MainThread
     public void loadUri(Uri uri) {
-        String path = uri.getPath();
+        String path = uri.toString();
         if (TextUtils.equals(uri.getScheme(), "content")) {
             path = "file://"+ FileUtils.getPathFromURI(uri);
         }
@@ -1583,23 +1664,41 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
         final MediaWrapper mw = mMediaList.getMedia(index);
         if (mw == null)
             return;
+        if (mw.getType() == MediaWrapper.TYPE_VIDEO && isVideoPlaying())
+            mw.addFlags(MediaWrapper.MEDIA_VIDEO);
 
         /* Pausable and seekable are true by default */
         mParsed = false;
+        mSwitchingToVideo = false;
         mPausable = mSeekable = true;
         final Media media = new Media(VLCInstance.get(), mw.getUri());
         VLCOptions.setMediaOptions(media, this, flags | mw.getFlags());
         media.setEventListener(mMediaListener);
         mMediaPlayer.setMedia(media);
         media.release();
-        mMediaPlayer.setEqualizer(VLCOptions.getEqualizer(this));
-        mMediaPlayer.setVideoTitleDisplay(MediaPlayer.Position.Disable, 0);
-        changeAudioFocus(true);
-        mMediaPlayer.setEventListener(mMediaPlayerListener);
-        mMediaPlayer.play();
+        if (mw .getType() != MediaWrapper.TYPE_VIDEO || mw.hasFlag(MediaWrapper.MEDIA_FORCE_AUDIO) || isVideoPlaying()) {
+            mMediaPlayer.setEqualizer(VLCOptions.getEqualizer(this));
+            mMediaPlayer.setVideoTitleDisplay(MediaPlayer.Position.Disable, 0);
+            changeAudioFocus(true);
+            mMediaPlayer.setEventListener(mMediaPlayerListener);
+            mMediaPlayer.play();
+            if(mSavedTime != 0l)
+                mMediaPlayer.setTime(mSavedTime);
+            mSavedTime = 0l;
 
-        notifyTrackChanged();
-        determinePrevAndNextIndices();
+            notifyTrackChanged();
+            determinePrevAndNextIndices();
+            if (mSettings.getBoolean(PreferencesFragment.PLAYBACK_HISTORY, true))
+                VLCApplication.runBackground(new Runnable() {
+                    @Override
+                    public void run() {
+                        MediaDatabase.getInstance().addHistoryItem(mw);
+                    }
+                });
+        } else {//Start VideoPlayer for first video, it will trigger playIndex when ready.
+            VideoPlayerActivity.startOpened(VLCApplication.getAppContext(),
+                    getCurrentMediaWrapper().getUri(), mCurrentIndex);
+        }
     }
 
     /**
@@ -1664,20 +1763,34 @@ public class PlaybackService extends Service implements IVLCVout.Callback {
     public void moveItem(int positionStart, int positionEnd) {
         mMediaList.move(positionStart, positionEnd);
         PlaybackService.this.saveMediaList();
-        Log.d(TAG, "moveItem "+positionStart+" -> "+positionEnd);
+        Log.d(TAG, "moveItem " + positionStart + " -> " + positionEnd);
     }
 
+    @MainThread
+    public void insertItem(int position, MediaWrapper mw) {
+        mMediaList.insert(position, mw);
+        saveMediaList();
+        determinePrevAndNextIndices();
+    }
+
+
     @MainThread
     public void remove(int position) {
         mMediaList.remove(position);
-        onMediaListChanged();
+        saveMediaList();
+        determinePrevAndNextIndices();
         Log.d(TAG, "remove "+position);
     }
 
     @MainThread
     public void removeLocation(String location) {
         mMediaList.remove(location);
-        onMediaListChanged();
+        saveMediaList();
+        determinePrevAndNextIndices();
+    }
+
+    public int getMediaListSize() {
+        return mMediaList.size();
     }
 
     @MainThread
