@@ -21,16 +21,24 @@ package org.videolan.vlc;
 
 import android.app.Application;
 import android.content.Context;
+import android.content.Intent;
 import android.content.SharedPreferences;
 import android.content.res.Configuration;
 import android.content.res.Resources;
 import android.preference.PreferenceManager;
+import android.support.v4.app.DialogFragment;
+import android.support.v4.util.SimpleArrayMap;
 import android.util.Log;
 
+import org.videolan.libvlc.Dialog;
+import org.videolan.vlc.gui.DialogActivity;
+import org.videolan.vlc.gui.dialogs.VlcProgressDialog;
 import org.videolan.vlc.gui.helpers.AudioUtil;
 import org.videolan.vlc.gui.helpers.BitmapCache;
 import org.videolan.vlc.media.MediaDatabase;
+import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.Strings;
+import org.videolan.vlc.util.VLCInstance;
 
 import java.util.Calendar;
 import java.util.Locale;
@@ -45,16 +53,25 @@ public class VLCApplication extends Application {
     public final static String SLEEP_INTENT = Strings.buildPkgString("SleepIntent");
 
     public static Calendar sPlayerSleepTime = null;
+
+    private static boolean sTV;
+    private static SharedPreferences mSettings;
+
+    private static SimpleArrayMap<String, Object> sDataMap = new SimpleArrayMap<>();
+
     /* Up to 2 threads maximum, inactive threads are killed after 2 seconds */
     private ThreadPoolExecutor mThreadPool = new ThreadPoolExecutor(0, 2, 2, TimeUnit.SECONDS,
                                                                     new LinkedBlockingQueue<Runnable>());
+
+    private static int sDialogCounter = 0;
+
     @Override
     public void onCreate() {
         super.onCreate();
 
         // Are we using advanced debugging - locale?
-        SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
-        String p = pref.getString("set_locale", "");
+        mSettings = PreferenceManager.getDefaultSharedPreferences(this);
+        String p = mSettings.getString("set_locale", "");
         if (!p.equals("")) {
             Locale locale;
             // workaround due to region code
@@ -89,6 +106,14 @@ public class VLCApplication extends Application {
         MediaDatabase.getInstance();
         // Prepare cache folder constants
         AudioUtil.prepareCacheFolder(this);
+
+        sTV = AndroidDevices.isAndroidTv() || !AndroidDevices.hasTsp();
+
+        Dialog.setCallbacks(VLCInstance.get(), mDialogCallbacks);
+
+        // Disable remote control receiver on Fire TV.
+        if (!AndroidDevices.hasTsp())
+            AndroidDevices.setRemoteControlReceiverEnabled(false);
     }
 
     /**
@@ -102,6 +127,14 @@ public class VLCApplication extends Application {
         BitmapCache.getInstance().clear();
     }
 
+    @Override
+    public void onTrimMemory(int level) {
+        super.onTrimMemory(level);
+        Log.w(TAG, "onTrimMemory, level: "+level);
+
+        BitmapCache.getInstance().clear();
+    }
+
     /**
      * @return the main context of the Application
      */
@@ -118,7 +151,66 @@ public class VLCApplication extends Application {
         return instance.getResources();
     }
 
+    public static boolean showTvUi() {
+        return sTV || mSettings.getBoolean("tv_ui", false);
+    }
+
     public static void runBackground(Runnable runnable) {
         instance.mThreadPool.execute(runnable);
     }
+
+    public static boolean removeTask(Runnable runnable) {
+        return instance.mThreadPool.remove(runnable);
+    }
+
+    public static void storeData(String key, Object data) {
+        sDataMap.put(key, data);
+    }
+
+    public static Object getData(String key) {
+        return sDataMap.remove(key);
+    }
+
+    Dialog.Callbacks mDialogCallbacks = new Dialog.Callbacks() {
+        @Override
+        public void onDisplay(Dialog.ErrorMessage dialog) {
+            Log.w(TAG, "ErrorMessage "+dialog.getText());
+        }
+
+        @Override
+        public void onDisplay(Dialog.LoginDialog dialog) {
+            String key = DialogActivity.KEY_LOGIN + sDialogCounter++;
+            fireDialog(dialog, key);
+        }
+
+        @Override
+        public void onDisplay(Dialog.QuestionDialog dialog) {
+            String key = DialogActivity.KEY_QUESTION + sDialogCounter++;
+            fireDialog(dialog, key);
+        }
+
+        @Override
+        public void onDisplay(Dialog.ProgressDialog dialog) {
+            String key = DialogActivity.KEY_PROGRESS + sDialogCounter++;
+            fireDialog(dialog, key);
+        }
+
+        @Override
+        public void onCanceled(Dialog dialog) {
+            ((DialogFragment)dialog.getContext()).dismiss();
+        }
+
+        @Override
+        public void onProgressUpdate(Dialog.ProgressDialog dialog) {
+            VlcProgressDialog vlcProgressDialog = (VlcProgressDialog) dialog.getContext();
+            if (vlcProgressDialog != null && vlcProgressDialog.isVisible())
+                vlcProgressDialog.updateProgress();
+        }
+    };
+
+    private void fireDialog(Dialog dialog, String key) {
+        storeData(key, dialog);
+        startActivity(new Intent(instance, DialogActivity.class).setAction(key)
+                .addFlags(Intent.FLAG_ACTIVITY_NEW_TASK));
+    }
 }
