@@ -22,20 +22,24 @@ package org.videolan.vlc.gui;
 
 import android.annotation.TargetApi;
 import android.app.SearchManager;
+import android.content.ComponentName;
 import android.content.Context;
 import android.content.Intent;
+import android.content.ServiceConnection;
 import android.content.SharedPreferences;
 import android.content.SharedPreferences.Editor;
 import android.content.pm.PackageManager;
+import android.content.res.Resources;
 import android.database.Cursor;
+import android.graphics.drawable.Drawable;
 import android.net.Uri;
 import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
+import android.os.IBinder;
 import android.os.Message;
 import android.support.design.widget.NavigationView;
 import android.support.v4.app.Fragment;
-import android.support.v4.app.FragmentActivity;
 import android.support.v4.app.FragmentManager;
 import android.support.v4.app.FragmentTransaction;
 import android.support.v4.view.GravityCompat;
@@ -43,11 +47,11 @@ import android.support.v4.view.MenuItemCompat;
 import android.support.v7.app.ActionBarDrawerToggle;
 import android.support.v7.widget.SearchView;
 import android.text.TextUtils;
-import android.util.Log;
 import android.view.KeyEvent;
 import android.view.Menu;
 import android.view.MenuInflater;
 import android.view.MenuItem;
+import android.view.SubMenu;
 import android.view.View;
 import android.view.Window;
 import android.view.WindowManager;
@@ -60,9 +64,14 @@ import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.vlc.BuildConfig;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
+import org.videolan.vlc.StartActivity;
 import org.videolan.vlc.VLCApplication;
+import org.videolan.vlc.extensions.ExtensionListing;
+import org.videolan.vlc.extensions.ExtensionManagerService;
+import org.videolan.vlc.extensions.api.VLCExtensionItem;
 import org.videolan.vlc.gui.audio.AudioBrowserFragment;
 import org.videolan.vlc.gui.browser.BaseBrowserFragment;
+import org.videolan.vlc.gui.browser.ExtensionBrowser;
 import org.videolan.vlc.gui.browser.FileBrowserFragment;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
 import org.videolan.vlc.gui.browser.NetworkBrowserFragment;
@@ -70,10 +79,11 @@ import org.videolan.vlc.gui.helpers.SearchSuggestionsAdapter;
 import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.network.MRLPanelFragment;
 import org.videolan.vlc.gui.preferences.PreferencesActivity;
+import org.videolan.vlc.gui.preferences.PreferencesFragment;
 import org.videolan.vlc.gui.video.VideoGridFragment;
 import org.videolan.vlc.gui.video.VideoListAdapter;
-import org.videolan.vlc.gui.video.VideoPlayerActivity;
 import org.videolan.vlc.gui.view.HackyDrawerLayout;
+import org.videolan.vlc.interfaces.IHistory;
 import org.videolan.vlc.interfaces.IRefreshable;
 import org.videolan.vlc.interfaces.ISortable;
 import org.videolan.vlc.media.MediaDatabase;
@@ -84,13 +94,17 @@ import org.videolan.vlc.util.Util;
 import org.videolan.vlc.util.VLCInstance;
 import org.videolan.vlc.util.WeakHandler;
 
-public class MainActivity extends AudioPlayerContainerActivity implements SearchSuggestionsAdapter.SuggestionDisplay, FilterQueryProvider, NavigationView.OnNavigationItemSelectedListener {
+import java.util.ArrayList;
+import java.util.List;
+
+public class MainActivity extends AudioPlayerContainerActivity implements SearchSuggestionsAdapter.SuggestionDisplay, FilterQueryProvider, NavigationView.OnNavigationItemSelectedListener, ExtensionManagerService.ExtensionManagerActivity {
     public final static String TAG = "VLC/MainActivity";
 
     private static final String PREF_FIRST_RUN = "first_run";
 
     private static final int ACTIVITY_RESULT_PREFERENCES = 1;
     private static final int ACTIVITY_RESULT_OPEN = 2;
+    public static final int ACTIVITY_RESULT_SECONDARY = 3;
     private static final int ACTIVITY_SHOW_INFOLAYOUT = 2;
     private static final int ACTIVITY_SHOW_PROGRESSBAR = 3;
     private static final int ACTIVITY_HIDE_PROGRESSBAR = 4;
@@ -106,7 +120,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
     private View mInfoLayout;
     private ProgressBar mInfoProgress;
     private TextView mInfoText;
-    private int mCurrentFragment;
+    private int mCurrentFragmentId;
 
 
     private int mVersionNumber = -1;
@@ -119,6 +133,11 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
     Menu mMenu;
     private SearchView mSearchView;
 
+    // Extensions management
+    private ServiceConnection mExtensionServiceConnection;
+    private ExtensionManagerService mExtensionManagerService;
+    private static final int PLUGIN_NAVIGATION_GROUP = 2;
+
     @Override
     protected void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
@@ -148,7 +167,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
             if (mSettings.getBoolean(PreferencesActivity.AUTO_RESCAN, true))
                 mMediaLibrary.scanMediaItems();
             else
-                mMediaLibrary.loadMedaItems();
+                mMediaLibrary.loadMediaItems();
         }
 
         /*** Start initializing the UI ***/
@@ -161,9 +180,9 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
         initAudioPlayerContainerActivity();
 
         if (savedInstanceState != null){
-            mCurrentFragment = savedInstanceState.getInt("current");
-            if (mCurrentFragment > 0)
-                mNavigationView.setCheckedItem(mCurrentFragment);
+            mCurrentFragmentId = savedInstanceState.getInt("current");
+            if (mCurrentFragmentId > 0)
+                mNavigationView.setCheckedItem(mCurrentFragmentId);
         }
 
 
@@ -217,10 +236,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
             item.setTitle(R.string.open);
         }
 
-        if (BuildConfig.DEBUG) {
-            mNavigationView.getMenu().findItem(R.id.nav_history).setVisible(true);
-            mNavigationView.getMenu().findItem(R.id.nav_network).setVisible(true);
-        }
+        mNavigationView.getMenu().findItem(R.id.nav_history).setVisible(mSettings.getBoolean(PreferencesFragment.PLAYBACK_HISTORY, true));
+
 
         if (AndroidUtil.isLolliPopOrLater())
             mNavigationView.setPadding(0, mNavigationView.getPaddingTop()/2, 0, 0);
@@ -257,6 +274,79 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
         mActionBar.setHomeButtonEnabled(true);
     }
 
+    @Override
+    protected void onStart() {
+        super.onStart();
+
+        createExtensionServiceConnection();
+
+        cleatBackstackFromExtension();
+    }
+
+    @Override
+    protected void onStop() {
+        super.onStop();
+        if (mExtensionServiceConnection != null) {
+            unbindService(mExtensionServiceConnection);
+            mExtensionServiceConnection = null;
+        }
+    }
+
+    private void loadPlugins() {
+        Menu navMenu = mNavigationView.getMenu();
+        navMenu.removeGroup(PLUGIN_NAVIGATION_GROUP);
+        List<ExtensionListing> plugins = mExtensionManagerService.updateAvailableExtensions();
+        if (plugins.isEmpty()) {
+            unbindService(mExtensionServiceConnection);
+            mExtensionServiceConnection = null;
+            mExtensionManagerService.stopSelf();
+            return;
+        }
+        PackageManager pm = getPackageManager();
+            SubMenu subMenu = navMenu.addSubMenu(PLUGIN_NAVIGATION_GROUP, PLUGIN_NAVIGATION_GROUP,
+               PLUGIN_NAVIGATION_GROUP, R.string.plugins);
+        for (int i = 0 ; i < plugins.size() ; ++i) {
+            ExtensionListing extension = plugins.get(i);
+            MenuItem item = subMenu.add(PLUGIN_NAVIGATION_GROUP, i, 0, extension.title());
+            int iconRes = extension.menuIcon();
+            Drawable extensionIcon = null;
+            if (iconRes != 0) {
+                try {
+                    Resources res = VLCApplication.getAppContext().getPackageManager().getResourcesForApplication(extension.componentName().getPackageName());
+                    extensionIcon = res.getDrawable(extension.menuIcon());
+                } catch (PackageManager.NameNotFoundException e) {}
+            }
+            if (extensionIcon != null)
+                item.setIcon(extensionIcon);
+            else
+                try {
+                    item.setIcon(pm.getApplicationIcon(plugins.get(i).componentName().getPackageName()));
+                } catch (PackageManager.NameNotFoundException e) {
+                    item.setIcon(R.drawable.icon);
+                }
+        }
+        mNavigationView.invalidate();
+    }
+
+    private void createExtensionServiceConnection() {
+        mExtensionServiceConnection = new ServiceConnection() {
+
+            @Override
+            public void onServiceConnected(ComponentName name, IBinder service) {
+                mExtensionManagerService = ((ExtensionManagerService.LocalBinder)service).getService();
+                mExtensionManagerService.setExtensionManagerActivity(MainActivity.this);
+                loadPlugins();
+            }
+
+            @Override
+            public void onServiceDisconnected(ComponentName name) {}
+        };
+        // Bind service which discoverves au connects toplugins
+        if (!bindService(new Intent(MainActivity.this,
+                ExtensionManagerService.class), mExtensionServiceConnection, Context.BIND_AUTO_CREATE))
+            mExtensionServiceConnection = null;
+    }
+
     @Override
     protected void onResume() {
         super.onResume();
@@ -266,8 +356,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
             mMediaLibrary.scanMediaItems();
         if (mSlidingPane.getState() == mSlidingPane.STATE_CLOSED)
             mActionBar.hide();
-        mNavigationView.setCheckedItem(mCurrentFragment);
-        mCurrentFragment = mSettings.getInt("fragment_id", R.id.nav_video);
+        mNavigationView.setCheckedItem(mCurrentFragmentId);
+        mCurrentFragmentId = mSettings.getInt("fragment_id", R.id.nav_video);
     }
 
     @Override
@@ -291,10 +381,10 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
          * (i.e. tracks) and replace it with a blank screen. (stuck menu bug)
          */
         if (current == null) {
-            mNavigationView.setCheckedItem(mCurrentFragment);
-            Fragment ff = getFragment(mCurrentFragment);
+            mNavigationView.setCheckedItem(mCurrentFragmentId);
+            Fragment ff = getFragment(mCurrentFragmentId);
             FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
-            ft.replace(R.id.fragment_placeholder, ff, getTag(mCurrentFragment));
+            ft.replace(R.id.fragment_placeholder, ff, getTag(mCurrentFragmentId));
             ft.commit();
         }
     }
@@ -311,7 +401,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
         mMediaLibrary.stop();
         /* Save the tab status in pref */
         SharedPreferences.Editor editor = mSettings.edit();
-        editor.putInt("fragment_id", mCurrentFragment);
+        editor.putInt("fragment_id", mCurrentFragmentId);
         Util.commitPreferences(editor);
 
         mFocusedPrior = 0;
@@ -319,7 +409,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
 
     protected void onSaveInstanceState(Bundle outState) {
         super.onSaveInstanceState(outState);
-        outState.putInt("current", mCurrentFragment);
+        outState.putInt("current", mCurrentFragmentId);
     }
 
     @Override
@@ -339,18 +429,19 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
             return;
         }
 
-        /* Slide down the audio player if it is shown entirely. */
-        if (slideDownAudioPlayer())
+        /* Close playlist search if open or Slide down the audio player if it is shown entirely. */
+        if (mAudioPlayer.clearSearch() || slideDownAudioPlayer())
             return;
 
         // If it's the directory view, a "backpressed" action shows a parent.
-        if (mCurrentFragment == R.id.nav_network || mCurrentFragment == R.id.nav_directories){
-            BaseBrowserFragment browserFragment = (BaseBrowserFragment) getSupportFragmentManager()
+        Fragment fragment = getSupportFragmentManager()
                     .findFragmentById(R.id.fragment_placeholder);
-            if (browserFragment != null) {
-                browserFragment.goBack();
-                return;
-            }
+        if (fragment instanceof BaseBrowserFragment){
+            ((BaseBrowserFragment)fragment).goBack();
+            return;
+        } else if (fragment instanceof ExtensionBrowser) {
+            ((ExtensionBrowser) fragment).goBack();
+            return;
         }
         finish();
     }
@@ -376,29 +467,32 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
         }
     }
 
-    private static void ShowFragment(FragmentActivity activity, String tag, Fragment fragment, String previous) {
-        if (fragment == null) {
-            Log.e(TAG, "Cannot show a null fragment, ShowFragment("+tag+") aborted.");
-            return;
+    @Override
+    public void displayExtensionItems(String title, List<VLCExtensionItem> items, boolean showParams, boolean refresh) {
+        FragmentManager fm = getSupportFragmentManager();
+
+        if (refresh && fm.findFragmentById(R.id.fragment_placeholder) instanceof ExtensionBrowser) {
+            ExtensionBrowser browser = (ExtensionBrowser) fm.findFragmentById(R.id.fragment_placeholder);
+            browser.doRefresh(title, items);
+        } else {
+            ExtensionBrowser fragment = new ExtensionBrowser();
+            ArrayList<VLCExtensionItem> list = new ArrayList<>(items);
+            Bundle args = new Bundle();
+            args.putParcelableArrayList(ExtensionBrowser.KEY_ITEMS_LIST, list);
+            args.putBoolean(ExtensionBrowser.KEY_SHOW_FAB, showParams);
+            args.putString(ExtensionBrowser.KEY_TITLE, title);
+            fragment.setArguments(args);
+            fragment.setExtensionService(mExtensionManagerService);
+
+            FragmentTransaction ft = fm.beginTransaction();
+            ft.setCustomAnimations(R.anim.anim_enter_right, 0, R.anim.anim_enter_left, 0);
+            ft.replace(R.id.fragment_placeholder, fragment, title);
+            if (!(fm.findFragmentById(R.id.fragment_placeholder) instanceof ExtensionBrowser))
+                ft.addToBackStack(getTag(mCurrentFragmentId));
+            else
+                ft.addToBackStack(title);
+            ft.commit();
         }
-
-        FragmentManager fm = activity.getSupportFragmentManager();
-
-        //abort if fragment is already the current one
-        Fragment current = fm.findFragmentById(R.id.fragment_placeholder);
-        if(current != null && current.getTag().equals(tag))
-            return;
-
-        //try to pop back if the fragment is already on the backstack
-        if (fm.popBackStackImmediate(tag, 0))
-            return;
-
-        //fragment is not there yet, spawn a new one
-        FragmentTransaction ft = fm.beginTransaction();
-        ft.setCustomAnimations(R.anim.anim_enter_right, R.anim.anim_leave_left, R.anim.anim_enter_left, R.anim.anim_leave_right);
-        ft.replace(R.id.fragment_placeholder, fragment, tag);
-        ft.addToBackStack(previous);
-        ft.commit();
     }
 
     /**
@@ -413,7 +507,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
         i.putExtra("fragment", fragmentTag);
         if (param != null)
             i.putExtra("param", param);
-        startActivity(i);
+        startActivityForResult(i, ACTIVITY_RESULT_SECONDARY);
         // Slide down the audio player if needed.
         slideDownAudioPlayer();
     }
@@ -483,18 +577,21 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
                 item.setTitle(R.string.sortby_date);
         }
 
-        boolean networkSave = current instanceof NetworkBrowserFragment && !((NetworkBrowserFragment)current).isRootDirectory();
-        if (networkSave) {
+        if (current instanceof NetworkBrowserFragment &&
+                !((NetworkBrowserFragment)current).isRootDirectory()) {
+            MenuItemCompat.setShowAsAction(menu.findItem(R.id.ml_menu_search), MenuItem.SHOW_AS_ACTION_IF_ROOM);
             item = menu.findItem(R.id.ml_menu_save);
             item.setVisible(true);
             String mrl = ((BaseBrowserFragment)current).mMrl;
-            item.setIcon(MediaDatabase.getInstance().networkFavExists(Uri.parse(mrl)) ?
+            boolean isFavorite = MediaDatabase.getInstance().networkFavExists(Uri.parse(mrl));
+            item.setIcon(isFavorite ?
                     R.drawable.ic_menu_bookmark_w :
                     R.drawable.ic_menu_bookmark_outline_w);
+            item.setTitle(isFavorite ? R.string.favorites_remove : R.string.favorites_add);
         } else
             menu.findItem(R.id.ml_menu_save).setVisible(false);
-        if (current instanceof MRLPanelFragment)
-            menu.findItem(R.id.ml_menu_clean).setVisible(!((MRLPanelFragment) current).isEmpty());
+        if (current instanceof IHistory)
+            menu.findItem(R.id.ml_menu_clean).setVisible(!((IHistory) current).isEmpty());
         boolean showLast = current instanceof AudioBrowserFragment || (current instanceof VideoGridFragment && mSettings.getString(PreferencesActivity.VIDEO_LAST, null) != null);
         menu.findItem(R.id.ml_menu_last_playlist).setVisible(showLast);
         return true;
@@ -535,14 +632,10 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
                 break;
             // Restore last playlist
             case R.id.ml_menu_last_playlist:
-                if (current instanceof AudioBrowserFragment) {
-                    Intent i = new Intent(PlaybackService.ACTION_REMOTE_LAST_PLAYLIST);
+                boolean audio = current instanceof AudioBrowserFragment;
+                    Intent i = new Intent(audio ? PlaybackService.ACTION_REMOTE_LAST_PLAYLIST :
+                           PlaybackService.ACTION_REMOTE_LAST_VIDEO_PLAYLIST);
                     sendBroadcast(i);
-                } else if (current instanceof VideoGridFragment) {
-                    final Uri uri = Uri.parse(mSettings.getString(PreferencesActivity.VIDEO_LAST, null));
-                    if (uri != null)
-                        VideoPlayerActivity.start(this, uri);
-                }
                 break;
             case android.R.id.home:
                 // Slide down the audio player.
@@ -554,8 +647,8 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
                 }
                 break;
             case R.id.ml_menu_clean:
-                if (current instanceof MRLPanelFragment)
-                    ((MRLPanelFragment)current).clearHistory();
+                if (current instanceof IHistory)
+                    ((IHistory)current).clearHistory();
                 break;
             case R.id.ml_menu_save:
                 if (current == null)
@@ -568,6 +661,10 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
         return super.onOptionsItemSelected(item);
     }
 
+    public void forceRefresh() {
+        forceRefresh(getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder));
+    }
+
     private void forceRefresh(Fragment current) {
         if (!mMediaLibrary.isWorking()) {
             if(current != null && current instanceof IRefreshable)
@@ -584,14 +681,16 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
             if (resultCode == PreferencesActivity.RESULT_RESCAN)
                 mMediaLibrary.scanMediaItems(true);
             else if (resultCode == PreferencesActivity.RESULT_RESTART) {
-                final Intent intent = getIntent();
-                if (intent != null && intent.getAction() != null) {
-                    finish();
-                    startActivity(intent);
-                }
+                Intent intent = new Intent(MainActivity.this, StartActivity.class);
+                finish();
+                startActivity(intent);
             }
         } else if (requestCode == ACTIVITY_RESULT_OPEN && resultCode == RESULT_OK){
             MediaUtils.openUri(this, data.getData());
+        } else if (requestCode == ACTIVITY_RESULT_SECONDARY) {
+            if (resultCode == PreferencesActivity.RESULT_RESCAN) {
+                forceRefresh(getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder));
+            }
         }
     }
 
@@ -704,7 +803,7 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
     }
 
     private void reloadPreferences() {
-        mCurrentFragment = mSettings.getInt("fragment_id", R.id.nav_video);
+        mCurrentFragmentId = mSettings.getInt("fragment_id", R.id.nav_video);
     }
 
     @Override
@@ -794,61 +893,80 @@ public class MainActivity extends AudioPlayerContainerActivity implements Search
         removeTipViewIfDisplayed();
     }
 
-
     @Override
     public boolean onNavigationItemSelected(MenuItem item) {
-        int id = item.getItemId();
-        Fragment current = getSupportFragmentManager().findFragmentById(R.id.fragment_placeholder);
-
-        if(current == null || (item != null && mCurrentFragment == id)) { /* Already selected */
-            if (mFocusedPrior != 0)
-                requestFocusOnSearch();
-            mDrawerLayout.closeDrawer(mNavigationView);
-            return false;
-        }
-
         // This should not happen
         if(item == null)
             return false;
 
-        String tag = getTag(id);
-        switch (id){
-            case R.id.nav_about:
-                showSecondaryFragment(SecondaryActivity.ABOUT);
-                break;
-            case R.id.nav_settings:
-                startActivityForResult(new Intent(this, PreferencesActivity.class), ACTIVITY_RESULT_PREFERENCES);
-                break;
-            case R.id.nav_directories:
-                if (TextUtils.equals(BuildConfig.FLAVOR_target, "chrome")) {
-                    Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
-                    intent.setType("audio/* video/*");
-                    startActivityForResult(intent, ACTIVITY_RESULT_OPEN);
-                    mDrawerLayout.closeDrawer(mNavigationView);
-                    return true;
-                }
-            default:
-                /* Slide down the audio player */
-                slideDownAudioPlayer();
+        int id = item.getItemId();
+        FragmentManager fm = getSupportFragmentManager();
+        Fragment current = fm.findFragmentById(R.id.fragment_placeholder);
 
-                /* Switch the fragment */
-                Fragment fragment = getFragment(id);
-                if (fragment instanceof MediaBrowserFragment)
-                    ((MediaBrowserFragment)fragment).setReadyToDisplay(false);
-                FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
-                ft.replace(R.id.fragment_placeholder, fragment, tag);
-                ft.addToBackStack(getTag(mCurrentFragment));
-                ft.commit();
-                mCurrentFragment = id;
+        if (item.getGroupId() == PLUGIN_NAVIGATION_GROUP)  {
+            mExtensionManagerService.openExtension(id);
+            mCurrentFragmentId = id;
+        } else {
+            if (mExtensionServiceConnection != null)
+                mExtensionManagerService.disconnect();
 
+            if(current == null || (item != null && mCurrentFragmentId == id)) { /* Already selected */
                 if (mFocusedPrior != 0)
                     requestFocusOnSearch();
+                mDrawerLayout.closeDrawer(mNavigationView);
+                return false;
+            }
+
+            String tag = getTag(id);
+            switch (id){
+                case R.id.nav_about:
+                    showSecondaryFragment(SecondaryActivity.ABOUT);
+                    break;
+                case R.id.nav_settings:
+                    startActivityForResult(new Intent(this, PreferencesActivity.class), ACTIVITY_RESULT_PREFERENCES);
+                    break;
+                case R.id.nav_directories:
+                    if (TextUtils.equals(BuildConfig.FLAVOR_target, "chrome")) {
+                        Intent intent = new Intent(Intent.ACTION_GET_CONTENT);
+                        intent.setType("audio/* video/*");
+                        startActivityForResult(intent, ACTIVITY_RESULT_OPEN);
+                        mDrawerLayout.closeDrawer(mNavigationView);
+                        return true;
+                    }
+                default:
+                /* Slide down the audio player */
+                    slideDownAudioPlayer();
+
+                /* Switch the fragment */
+                    Fragment fragment = getFragment(id);
+                    if (fragment instanceof MediaBrowserFragment)
+                        ((MediaBrowserFragment)fragment).setReadyToDisplay(false);
+                    FragmentTransaction ft = fm.beginTransaction();
+                    ft.replace(R.id.fragment_placeholder, fragment, tag);
+                    ft.addToBackStack(getTag(mCurrentFragmentId));
+                    ft.commit();
+                    mCurrentFragmentId = id;
+
+                    if (mFocusedPrior != 0)
+                        requestFocusOnSearch();
+            }
         }
+        mNavigationView.setCheckedItem(mCurrentFragmentId);
         mDrawerLayout.closeDrawer(mNavigationView);
-        mNavigationView.setCheckedItem(mCurrentFragment);
         return true;
     }
 
+    private void cleatBackstackFromExtension() {
+        FragmentManager fm = getSupportFragmentManager();
+        Fragment current = getSupportFragmentManager()
+                .findFragmentById(R.id.fragment_placeholder);
+        while (current instanceof ExtensionBrowser) {
+            fm.popBackStackImmediate();
+            current = getSupportFragmentManager()
+                    .findFragmentById(R.id.fragment_placeholder);
+        }
+    }
+
     private String getTag(int id){
         switch (id){
             case R.id.nav_about:
