@@ -26,8 +26,7 @@ import android.content.Intent;
 import android.os.Build;
 import android.os.Bundle;
 import android.os.Handler;
-import android.os.Message;
-import android.support.design.widget.Snackbar;
+import android.os.Looper;
 import android.support.design.widget.TabLayout;
 import android.support.v4.view.ViewPager;
 import android.view.ContextMenu;
@@ -51,17 +50,19 @@ import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
+import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.PlaybackServiceFragment;
 import org.videolan.vlc.gui.SecondaryActivity;
 import org.videolan.vlc.gui.helpers.AudioUtil;
 import org.videolan.vlc.gui.helpers.MediaComparators;
+import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.view.SwipeRefreshLayout;
+import org.videolan.vlc.media.MediaDatabase;
 import org.videolan.vlc.media.MediaLibrary;
 import org.videolan.vlc.media.MediaUtils;
 import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.AndroidDevices;
 import org.videolan.vlc.util.FileUtils;
-import org.videolan.vlc.util.WeakHandler;
 
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -72,11 +73,9 @@ public class AudioAlbumsSongsFragment extends PlaybackServiceFragment implements
 
     public final static String TAG = "VLC/AudioAlbumsSongsFragment";
 
-    private static final int DELETE_MEDIA = 0;
-    private static final int DELETE_DURATION = 3000;
-
     private MediaLibrary mMediaLibrary;
     private PlaybackService.Client mClient;
+    Handler mHandler = new Handler(Looper.getMainLooper());
 
     private SwipeRefreshLayout mSwipeRefreshLayout;
     private ViewPager mViewPager;
@@ -200,9 +199,9 @@ public class AudioAlbumsSongsFragment extends PlaybackServiceFragment implements
             menu.setGroupVisible(R.id.phone_only, false);
         menu.findItem(R.id.audio_list_browser_play).setVisible(true);
         //Hide delete if we cannot
-            AudioBrowserListAdapter adapter = mViewPager.getCurrentItem() == MODE_SONG ?
-                    mSongsAdapter : mAlbumsAdapter;
-            String location = adapter.getItem(position).mMediaList.get(0).getLocation();
+        AudioBrowserListAdapter adapter = mViewPager.getCurrentItem() == MODE_SONG ?
+                mSongsAdapter : mAlbumsAdapter;
+        String location = adapter.getItem(position).mMediaList.get(0).getLocation();
         menu.findItem(R.id.audio_list_browser_delete).setVisible(
                 FileUtils.canWrite(location));
     }
@@ -218,21 +217,31 @@ public class AudioAlbumsSongsFragment extends PlaybackServiceFragment implements
         return super.onContextItemSelected(menu);
     }
 
-    private boolean handleContextItemSelected(MenuItem item, int position) {
+    private boolean handleContextItemSelected(MenuItem item, final int position) {
 
         int startPosition;
         List<MediaWrapper> medias;
         int id = item.getItemId();
+        final AudioBrowserListAdapter adapter = mViewPager.getCurrentItem() == MODE_ALBUM ? mAlbumsAdapter : mSongsAdapter;
 
         boolean useAllItems = id == R.id.audio_list_browser_play_all;
         boolean append = id == R.id.audio_list_browser_append;
 
         if (id == R.id.audio_list_browser_delete) {
-            Snackbar.make(getView(), getString(R.string.file_deleted), Snackbar.LENGTH_LONG)
-                .setAction(android.R.string.cancel, mCancelDeleteMediaListener)
-                .show();
-                Message msg = mHandler.obtainMessage(DELETE_MEDIA, position, 0);
-            mHandler.sendMessageDelayed(msg, DELETE_DURATION);
+            final AudioBrowserListAdapter.ListItem listItem = adapter.getItem(position);
+            final String key = adapter.getKey(position);
+            adapter.remove(position, key);
+            UiTools.snackerWithCancel(getView(), getString(R.string.file_deleted), new Runnable() {
+                @Override
+                public void run() {
+                    deleteMedia(listItem);
+                }
+            }, new Runnable() {
+                @Override
+                public void run() {
+                    adapter.addItem(position, key, listItem);
+                }
+            });
             return true;
         }
 
@@ -241,23 +250,21 @@ public class AudioAlbumsSongsFragment extends PlaybackServiceFragment implements
             return true;
         }
 
+        if (id == R.id.audio_view_info) {
+                Intent i = new Intent(getActivity(), SecondaryActivity.class);
+                i.putExtra("fragment", "mediaInfo");
+                i.putExtra("param", mSongsAdapter.getItem(position).mMediaList.get(0).getUri().toString());
+                getActivity().startActivityForResult(i, MainActivity.ACTIVITY_RESULT_SECONDARY);
+                return true;
+        }
+
         if (useAllItems) {
             medias = new ArrayList<MediaWrapper>();
             startPosition = mSongsAdapter.getListWithPosition(medias, position);
         }
         else {
             startPosition = 0;
-            switch (mViewPager.getCurrentItem())
-            {
-                case MODE_ALBUM: // albums
-                    medias = mAlbumsAdapter.getMedias(position);
-                    break;
-                case MODE_SONG: // songs
-                    medias = mSongsAdapter.getMedias(position);
-                    break;
-                default:
-                    return true;
-            }
+            medias = adapter.getMedias(position);
         }
 
         if (mService != null) {
@@ -305,8 +312,8 @@ public class AudioAlbumsSongsFragment extends PlaybackServiceFragment implements
         public void onItemClick(AdapterView<?> av, View v, int p, long id) {
             ArrayList<MediaWrapper> mediaList = mAlbumsAdapter.getMedias(p);
             Intent i = new Intent(getActivity(), SecondaryActivity.class);
-            i.putExtra("fragment", SecondaryActivity.ALBUM);
-            i.putParcelableArrayListExtra("list", mediaList);
+            i.putExtra(SecondaryActivity.KEY_FRAGMENT, SecondaryActivity.ALBUM);
+            VLCApplication.storeData(SecondaryActivity.ALBUM, mediaList);
             i.putExtra("filter", mAlbumsAdapter.getTitle(p));
             startActivity(i);
         }
@@ -370,41 +377,30 @@ public class AudioAlbumsSongsFragment extends PlaybackServiceFragment implements
         mSongsAdapter.clear();
     }
 
-
-    View.OnClickListener mCancelDeleteMediaListener = new View.OnClickListener() {
-        @Override
-        public void onClick(View view) {
-            mHandler.removeMessages(DELETE_MEDIA);
+    private void deleteMedia(final AudioBrowserListAdapter.ListItem listItem) {
+        for (final MediaWrapper media : listItem.mMediaList) {
+            mMediaList.remove(media);
+            mMediaLibrary.getMediaItems().remove(media);
+            mHandler.post(new Runnable() {
+                @Override
+                public void run() {
+                    if (mService != null)
+                        mService.removeLocation(media.getLocation());
+                }
+            });
+            VLCApplication.runBackground(new Runnable() {
+                @Override
+                public void run() {
+                    MediaDatabase.getInstance().removeMedia(media.getUri());
+                    FileUtils.deleteFile(media.getUri().getPath());
+                }
+            });
         }
-    };
-
-    Handler mHandler = new AudioAlbumsSongsFragmentHandler(this);
-
-    private static class AudioAlbumsSongsFragmentHandler extends WeakHandler<AudioAlbumsSongsFragment>{
-
-        public AudioAlbumsSongsFragmentHandler(AudioAlbumsSongsFragment owner) {
-            super(owner);
-        }
-
-        @Override
-        public void handleMessage(Message msg) {
-            super.handleMessage(msg);
-            final AudioAlbumsSongsFragment fragment = getOwner();
-            switch (msg.what){
-                case DELETE_MEDIA:
-                    AudioBrowserListAdapter.ListItem listItem = fragment.mSongsAdapter.getItem(msg.arg1);
-                    for (MediaWrapper media : listItem.mMediaList) {
-                        final String path = media.getUri().getPath();
-                        fragment.mMediaLibrary.getMediaItems().remove(media);
-                        fragment.mSongsAdapter.removeMedia(media);
-                        fragment.mAlbumsAdapter.removeMedia(media);
-                        if (fragment.mService != null)
-                            fragment.mService.removeLocation(media.getLocation());
-                        fragment.mMediaLibrary.getMediaItems().remove(media);
-                        FileUtils.asyncRecursiveDelete(path);
-                    }
-                    break;
+        mHandler.post(new Runnable() {
+            @Override
+            public void run() {
+                updateList();
             }
-        }
+        });
     }
 }
