@@ -27,7 +27,6 @@ import android.os.Bundle;
 import android.os.Handler;
 import android.os.Message;
 import android.support.design.widget.FloatingActionButton;
-import android.support.design.widget.Snackbar;
 import android.support.design.widget.TabLayout;
 import android.support.v4.view.ViewPager;
 import android.view.ContextMenu;
@@ -59,6 +58,7 @@ import org.videolan.vlc.gui.SecondaryActivity;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
 import org.videolan.vlc.gui.helpers.AudioUtil;
 import org.videolan.vlc.gui.helpers.MediaComparators;
+import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.view.SwipeRefreshLayout;
 import org.videolan.vlc.interfaces.IBrowser;
 import org.videolan.vlc.media.MediaDatabase;
@@ -97,9 +97,8 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
     private List<View> mLists;
     private FloatingActionButton mFabPlayShuffleAll;
 
-    public static final int DELETE_MEDIA = 101;
-    public static final int DELETE_PLAYLIST = 102;
-    public static final int DELETE_DURATION = 3000;
+    public static final int REFRESH = 101;
+    public static final int UPDATE_LIST = 102;
     public final static int MODE_ARTIST = 0;
     public final static int MODE_ALBUM = 1;
     public final static int MODE_SONG = 2;
@@ -161,6 +160,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
         mTabLayout = (TabLayout) v.findViewById(R.id.sliding_tabs);
         mTabLayout.setupWithViewPager(mViewPager);
 
+        setTabsClickListeners();
         mViewPager.addOnPageChangeListener(this);
 
         songsList.setOnItemClickListener(songListener);
@@ -204,20 +204,39 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
         return v;
     }
 
-    AbsListView.OnScrollListener mScrollListener = new AbsListView.OnScrollListener(){
-            @Override
-            public void onScrollStateChanged(AbsListView view, int scrollState) {
-                boolean enabled = scrollState == SCROLL_STATE_IDLE;
-                if (enabled) {
-                    enabled = view.getFirstVisiblePosition() == 0;
-                    if (view.getChildAt(0) != null)
-                        enabled &= view.getChildAt(0).getTop() == 0;
+    private void setTabsClickListeners() {
+        for (int i = 0; i < mTabLayout.getTabCount(); ++i){
+            final int position = i;
+            ((ViewGroup)mTabLayout.getChildAt(0)).getChildAt(i).setOnClickListener(new View.OnClickListener() {
+                @Override
+                public void onClick(View v) {
+                    if(position == mViewPager.getCurrentItem()){
+                        ((ListView)mLists.get(position)).smoothScrollToPosition(0);
+                    } else {
+                        mViewPager.setCurrentItem(position);
+                        updateEmptyView(position);
+                        setFabPlayShuffleAllVisibility();
+                        tcl.onPageSelected(position);
+                    }
                 }
-                mSwipeRefreshLayout.setEnabled(enabled);
+            });
+        }
+    }
+
+    AbsListView.OnScrollListener mScrollListener = new AbsListView.OnScrollListener(){
+        @Override
+        public void onScrollStateChanged(AbsListView view, int scrollState) {
+            boolean enabled = scrollState == SCROLL_STATE_IDLE;
+            if (enabled) {
+                enabled = view.getFirstVisiblePosition() == 0;
+                if (view.getChildAt(0) != null)
+                    enabled &= view.getChildAt(0).getTop() == 0;
             }
-            @Override
-            public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount,
-                                 int totalItemCount) {}
+            mSwipeRefreshLayout.setEnabled(enabled);
+        }
+        @Override
+        public void onScroll(AbsListView view, int firstVisibleItem, int visibleItemCount,
+                             int totalItemCount) {}
     };
 
     @Override
@@ -326,9 +345,9 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
                 return;
             MainActivity activity = (MainActivity)getActivity();
             Intent i = new Intent(getActivity(), SecondaryActivity.class);
-            i.putExtra("fragment", "albumsSongs");
-            i.putParcelableArrayListExtra("list", mediaList);
-            i.putExtra("filter", MediaUtils.getMediaArtist(activity, mediaList.get(0)));
+            i.putExtra(SecondaryActivity.KEY_FRAGMENT, SecondaryActivity.ALBUMS_SONGS);
+            VLCApplication.storeData(SecondaryActivity.ALBUMS_SONGS, mediaList);
+            i.putExtra(SecondaryActivity.KEY_FILTER, MediaUtils.getMediaArtist(activity, mediaList.get(0)));
             startActivity(i);
         }
     };
@@ -340,9 +359,9 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
             if (mediaList.isEmpty())
                 return;
             Intent i = new Intent(getActivity(), SecondaryActivity.class);
-            i.putExtra("fragment", SecondaryActivity.ALBUM);
-            i.putParcelableArrayListExtra("list", mediaList);
-            i.putExtra("filter", MediaUtils.getMediaAlbum(getActivity(), mediaList.get(0)));
+            i.putExtra(SecondaryActivity.KEY_FRAGMENT, SecondaryActivity.ALBUM);
+            VLCApplication.storeData(SecondaryActivity.ALBUM, mediaList);
+            i.putExtra(SecondaryActivity.KEY_FILTER, MediaUtils.getMediaAlbum(getActivity(), mediaList.get(0)));
             startActivity(i);
         }
     };
@@ -354,9 +373,9 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
             if (mediaList.isEmpty())
                 return;
             Intent i = new Intent(getActivity(), SecondaryActivity.class);
-            i.putExtra("fragment", SecondaryActivity.ALBUMS_SONGS);
-            i.putParcelableArrayListExtra("list", mediaList);
-            i.putExtra("filter", MediaUtils.getMediaGenre(getActivity(), mediaList.get(0)));
+            i.putExtra(SecondaryActivity.KEY_FRAGMENT, SecondaryActivity.ALBUMS_SONGS);
+            VLCApplication.storeData(SecondaryActivity.ALBUMS_SONGS, mediaList);
+            i.putExtra(SecondaryActivity.KEY_FILTER, MediaUtils.getMediaGenre(getActivity(), mediaList.get(0)));
             startActivity(i);
         }
     };
@@ -424,7 +443,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
         return super.onContextItemSelected(menu);
     }
 
-    private boolean handleContextItemSelected(MenuItem item, int position) {
+    private boolean handleContextItemSelected(final MenuItem item, final int position) {
 
         int startPosition;
         int mode = mViewPager.getCurrentItem();
@@ -435,7 +454,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
         boolean append = id == R.id.audio_list_browser_append;
 
         if (id == R.id.audio_list_browser_delete) {
-            AudioBrowserListAdapter adapter;
+            final AudioBrowserListAdapter adapter;
             if (mode == MODE_SONG){
                 adapter = mSongsAdapter;
             } else if (mode == MODE_PLAYLIST) {
@@ -445,19 +464,37 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
             List<MediaWrapper> mediaList = adapter.getMedias(position);
             if (adapter.getCount() <= position || mediaList == null || mediaList.isEmpty())
                 return false;
-            Message msg;
+            final MediaWrapper media = mediaList.get(0);
+            final AudioBrowserListAdapter.ListItem listItem = adapter.getItem(position);
+            final String key = adapter.getKey(position);
+            String message;
+            Runnable action;
+
+            adapter.remove(position, key);
+
             if (mode == MODE_PLAYLIST) {
-                Snackbar.make(getView(), getString(R.string.playlist_deleted), DELETE_DURATION)
-                .setAction(android.R.string.cancel, mCancelDeletePlayListListener)
-                .show();
-                msg = mHandler.obtainMessage(DELETE_PLAYLIST, position, 0);
+                message = getString(R.string.playlist_deleted);
+                action = new Runnable() {
+                    @Override
+                    public void run() {
+                        deletePlaylist(listItem);
+                    }
+                };
             } else {
-                Snackbar.make(getView(), getString(R.string.file_deleted), Snackbar.LENGTH_LONG)
-                        .setAction(android.R.string.cancel, mCancelDeleteMediaListener)
-                        .show();
-                msg = mHandler.obtainMessage(DELETE_MEDIA, position, 0);
+                message = getString(R.string.file_deleted);
+                action = new Runnable() {
+                    @Override
+                    public void run() {
+                        deleteMedia(media);
+                    }
+                };
             }
-            mHandler.sendMessageDelayed(msg, DELETE_DURATION);
+            UiTools.snackerWithCancel(getView(), message, action, new Runnable() {
+                @Override
+                public void run() {
+                    adapter.addItem(position, key, listItem);
+                }
+            });
             return true;
         }
 
@@ -468,6 +505,14 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
             return true;
         }
 
+        if (id == R.id.audio_view_info) {
+                Intent i = new Intent(getActivity(), SecondaryActivity.class);
+                i.putExtra("fragment", "mediaInfo");
+                i.putExtra("param", mSongsAdapter.getItem(position).mMediaList.get(0).getUri().toString());
+                getActivity().startActivityForResult(i, MainActivity.ACTIVITY_RESULT_SECONDARY);
+                return true;
+        }
+
         if (useAllItems) {
             if (mSongsAdapter.getCount() <= position)
                 return false;
@@ -494,7 +539,7 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
                     if (medias.size() == 1 && mPlaylistAdapter.getMedias(position).get(0).getType() == MediaWrapper.TYPE_PLAYLIST) {
                         if (mMediaBrowser == null)
                             mMediaBrowser = new MediaBrowser(VLCInstance.get(), this);
-                        mMediaBrowser.browse(mPlaylistAdapter.getMedias(position).get(0).getUri());
+                        mMediaBrowser.browse(mPlaylistAdapter.getMedias(position).get(0).getUri(), true);
                         return true;
                     } else
                         adapter = mPlaylistAdapter;
@@ -517,19 +562,6 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
             return false;
     }
 
-    View.OnClickListener mCancelDeletePlayListListener = new View.OnClickListener() {
-        @Override
-        public void onClick(View view) {
-            mHandler.removeMessages(DELETE_PLAYLIST);
-        }
-    };
-
-    View.OnClickListener mCancelDeleteMediaListener = new View.OnClickListener() {
-        @Override
-        public void onClick(View view) {
-            mHandler.removeMessages(DELETE_MEDIA);
-        }
-    };
     public void onFabPlayAllClick(View view) {
         List<MediaWrapper> medias = new ArrayList<MediaWrapper>();
         mSongsAdapter.getListWithPosition(medias, 0);
@@ -672,6 +704,43 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
         tcl.onPageSelected(position);
     }
 
+    private void deleteMedia(final MediaWrapper mw) {
+        VLCApplication.runBackground(new Runnable() {
+            @Override
+            public void run() {
+                final String path = mw.getUri().getPath();
+                FileUtils.deleteFile(path);
+                MediaDatabase.getInstance().removeMedia(mw.getUri());
+                mMediaLibrary.getMediaItems().remove(mw);
+                mHandler.post(new Runnable() {
+                    @Override
+                    public void run() {
+                        if (mService != null)
+                            mService.removeLocation(mw.getLocation());
+                    }
+                });
+                mHandler.obtainMessage(REFRESH, path).sendToTarget();
+            }
+        });
+    }
+
+    private void deletePlaylist(final AudioBrowserListAdapter.ListItem listItem) {
+        VLCApplication.runBackground(new Runnable() {
+            @Override
+            public void run() {
+                if (!MediaDatabase.getInstance().playlistExists(listItem.mTitle)) { //File playlist
+                    MediaWrapper media = listItem.mMediaList.get(0);
+                    mMediaLibrary.getMediaItems().remove(media);
+                    FileUtils.deleteFile(media.getUri().getPath());
+                    mHandler.obtainMessage(REFRESH, media.getLocation()).sendToTarget();
+                } else {
+                    MediaDatabase.getInstance().playlistDelete(listItem.mTitle);
+                }
+                mHandler.obtainMessage(UPDATE_LIST).sendToTarget();
+            }
+        });
+    }
+
     @Override
     public void onPageScrollStateChanged(int state) {
         tcl.onPageScrollStateChanged(state);
@@ -696,24 +765,10 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
                             fragment.mSongsAdapter.isEmpty() && fragment.mGenresAdapter.isEmpty())
                         fragment.mSwipeRefreshLayout.setRefreshing(true);
                     break;
-                case DELETE_MEDIA:
-                    MediaWrapper mw = fragment.mSongsAdapter.getItem(msg.arg1).mMediaList.get(0);
-                    final String path = mw.getUri().getPath();
-                    //Let's keep this toast while duration is not set correctly
-                    FileUtils.asyncRecursiveDelete(path);
-                    fragment.mMediaLibrary.getMediaItems().remove(mw);
-                    refresh(fragment, path);
+                case REFRESH:
+                    refresh(fragment, (String) msg.obj);
                     break;
-                case DELETE_PLAYLIST:
-                    AudioBrowserListAdapter.ListItem listItem = fragment.mPlaylistAdapter.getItem(msg.arg1);
-                    //Let's keep this toast while duration is not set correctly
-                    if (!MediaDatabase.getInstance().playlistExists(listItem.mTitle)) { //File playlist
-                        MediaWrapper media = listItem.mMediaList.get(0);
-                        fragment.mMediaLibrary.getMediaItems().remove(media);
-                        refresh(fragment, media.getLocation());
-                    } else {
-                        MediaDatabase.getInstance().playlistDelete(listItem.mTitle);
-                    }
+                case UPDATE_LIST:
                     fragment.updateLists();
                     break;
             }
@@ -745,7 +800,8 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
                     updateAlbums, updateSongs, updateGenres, updatePlaylists));
 
             //process the visible list first
-            tasks.add(0, tasks.remove(mViewPager.getCurrentItem()));
+            if (mViewPager.getCurrentItem() != 0)
+                tasks.add(0, tasks.remove(mViewPager.getCurrentItem()));
             tasks.add(new Runnable() {
                 @Override
                 public void run() {
@@ -824,30 +880,30 @@ public class AudioBrowserFragment extends MediaBrowserFragment implements SwipeR
     };
 
     AudioBrowserListAdapter.ContextPopupMenuListener mContextPopupMenuListener
-        = new AudioBrowserListAdapter.ContextPopupMenuListener() {
+            = new AudioBrowserListAdapter.ContextPopupMenuListener() {
 
-            @Override
-            @TargetApi(Build.VERSION_CODES.HONEYCOMB)
-            public void onPopupMenu(View anchor, final int position) {
-                if (!AndroidUtil.isHoneycombOrLater()) {
-                    // Call the "classic" context menu
-                    anchor.performLongClick();
-                    return;
-                }
-
-                PopupMenu popupMenu = new PopupMenu(getActivity(), anchor);
-                popupMenu.getMenuInflater().inflate(R.menu.audio_list_browser, popupMenu.getMenu());
-                setContextMenuItems(popupMenu.getMenu(), position);
-
-                popupMenu.setOnMenuItemClickListener(new OnMenuItemClickListener() {
-                    @Override
-                    public boolean onMenuItemClick(MenuItem item) {
-                        return handleContextItemSelected(item, position);
-                    }
-                });
-                popupMenu.show();
+        @Override
+        @TargetApi(Build.VERSION_CODES.HONEYCOMB)
+        public void onPopupMenu(View anchor, final int position) {
+            if (!AndroidUtil.isHoneycombOrLater()) {
+                // Call the "classic" context menu
+                anchor.performLongClick();
+                return;
             }
 
+            PopupMenu popupMenu = new PopupMenu(getActivity(), anchor);
+            popupMenu.getMenuInflater().inflate(R.menu.audio_list_browser, popupMenu.getMenu());
+            setContextMenuItems(popupMenu.getMenu(), position);
+
+            popupMenu.setOnMenuItemClickListener(new OnMenuItemClickListener() {
+                @Override
+                public boolean onMenuItemClick(MenuItem item) {
+                    return handleContextItemSelected(item, position);
+                }
+            });
+            popupMenu.show();
+        }
+
     };
 
     /*
