@@ -201,14 +201,31 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
         });
     }
 
+    public void remove(int position, String key) {
+        mItems.remove(position);
+        mMediaItemMap.remove(key);
+        notifyDataSetChanged();
+    }
+
+    public void addItem(int position, String key, ListItem item) {
+        mMediaItemMap.put(key, item);
+        mItems.add(position, item);
+        notifyDataSetChanged();
+    }
+
+    public String getKey(int position) {
+        return (String) mMediaItemMap.keySet().toArray()[position];
+    }
+
     /**
      * Calculate sections of the list
      *
      * @param type Type of the audio file sort.
      */
     private void calculateSections(int type) {
-        char prevFirstChar = 'a';
+        char prevFirstChar = '%';
         boolean firstSeparator = true;
+        ArrayList<String> sections = new ArrayList<>();
 
         for (int i = 0; i < mItems.size(); ++i) {
             String title = mItems.get(i).mTitle;
@@ -233,13 +250,15 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
                 firstChar = '#'; // Blank / spaces-only song title.
 
             if (Character.isLetter(firstChar)) {
-                if (firstSeparator || firstChar != prevFirstChar) {
-                    ListItem item = new ListItem(String.valueOf(firstChar), null, null, true);
+                String firstCharInString = String.valueOf(firstChar);
+                if ((firstSeparator || firstChar != prevFirstChar) && !sections.contains(firstCharInString)) {
+                    ListItem item = new ListItem(firstCharInString, null, null, true);
                     mItems.add(i, item);
                     mSections.put(i, String.valueOf(firstChar));
                     i++;
                     prevFirstChar = firstChar;
                     firstSeparator = false;
+                    sections.add(firstCharInString);
                 }
             }
             else if (firstSeparator) {
@@ -281,7 +300,7 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
      * Remove also all the list items that contain only this media.
      * @param media the media to remove
      */
-    public void removeMedia(MediaWrapper media) {
+    public void removeMedia(MediaWrapper media, boolean notify) {
         for (int i = 0; i < mItems.size(); ++i) {
             ListItem item = mItems.get(i);
             if (item.mMediaList == null)
@@ -296,7 +315,8 @@ public class AudioBrowserListAdapter extends BaseAdapter implements SectionIndex
                 i--;
             }
         }
-        notifyDataSetChanged();
+        if (notify)
+            notifyDataSetChanged();
     }
 
     public void clear() {
