@@ -25,22 +25,23 @@ package org.videolan.vlc.gui.browser;
 import android.databinding.DataBindingUtil;
 import android.net.Uri;
 import android.support.v7.widget.RecyclerView;
+import android.text.TextUtils;
 import android.view.LayoutInflater;
 import android.view.View;
 import android.view.ViewGroup;
-import android.widget.CheckBox;
-import android.widget.ImageView;
 
 import org.videolan.libvlc.Media;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.databinding.BrowserItemSeparatorBinding;
 import org.videolan.vlc.databinding.DirectoryViewItemBinding;
+import org.videolan.vlc.gui.helpers.AsyncImageLoader;
 import org.videolan.vlc.gui.helpers.MediaComparators;
 import org.videolan.vlc.media.MediaDatabase;
 import org.videolan.vlc.media.MediaUtils;
 import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.CustomDirectories;
+import org.videolan.vlc.util.HttpImageLoader;
 
 import java.io.File;
 import java.util.ArrayList;
@@ -49,7 +50,7 @@ import java.util.Collections;
 import java.util.LinkedList;
 import java.util.List;
 
-public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewHolder> implements View.OnLongClickListener {
+public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewHolder> {
     protected static final String TAG = "VLC/BaseBrowserAdapter";
 
     protected static final int TYPE_MEDIA = 0;
@@ -64,6 +65,7 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
     LinkedList<String> mMediaDirsLocation;
     List<String> mCustomDirsLocation;
     String mEmptyDirectoryString;
+    private int mTop = 0;
 
     public BaseBrowserAdapter(BaseBrowserFragment fragment){
         this.fragment = fragment;
@@ -100,20 +102,18 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
     private void onBindMediaViewHolder(final RecyclerView.ViewHolder holder, int position) {
         final MediaViewHolder vh = (MediaViewHolder) holder;
         final MediaWrapper media = (MediaWrapper) getItem(position);
-        boolean hasContextMenu = (media.getType() == MediaWrapper.TYPE_AUDIO ||
-                media.getType() == MediaWrapper.TYPE_VIDEO ||
-                media.getType() == MediaWrapper.TYPE_DIR );
-        vh.binding.setPosition(position);
-        vh.binding.setHandler(mClickHandler);
         vh.binding.setMedia(media);
-        vh.binding.setHasContextMenu(hasContextMenu);
         vh.binding.setType(TYPE_MEDIA);
+        vh.binding.setHasContextMenu(true);
+        if (fragment instanceof NetworkBrowserFragment && fragment.isRootDirectory())
+            vh.binding.setProtocole(getProtocol(media));
         vh.binding.executePendingBindings();
 
-        vh.icon.setImageResource(getIconResId(media));
-        if (hasContextMenu) {
-            vh.itemView.setOnLongClickListener(this);
-        }
+        vh.binding.dviIcon.setBackgroundResource(getIconResId(media));
+        if (!TextUtils.isEmpty(media.getArtworkURL()) && media.getArtworkURL().startsWith("http"))
+            AsyncImageLoader.LoadImage(new HttpImageLoader(media.getArtworkURL(), vh.binding), null);
+
+        vh.setContextMenuListener();
     }
 
     @Override
@@ -121,23 +121,60 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
         return mMediaList.size();
     }
 
-    public class MediaViewHolder extends RecyclerView.ViewHolder {
-        public CheckBox checkBox;
-        public ImageView icon;
-        public ImageView more;
+    public class ViewHolder extends RecyclerView.ViewHolder {
+
+        public ViewHolder(View itemView) {
+            super(itemView);
+        }
+        public void onClick(View v){}
+
+        public void onCheckBoxClick(View v){}
+
+        public void onMoreClick(View v){}
+    }
+
+    public class MediaViewHolder extends ViewHolder implements View.OnLongClickListener {
         DirectoryViewItemBinding binding;
 
         public MediaViewHolder(View v) {
             super(v);
             binding = DataBindingUtil.bind(v);
-            checkBox = (CheckBox) v.findViewById(R.id.browser_checkbox);
-            icon = (ImageView) v.findViewById(R.id.dvi_icon);
-            more = (ImageView) v.findViewById(R.id.item_more);
+            binding.setHolder(this);
             v.findViewById(R.id.layout_item).setTag(R.id.layout_item, this);
         }
+
+        public void setContextMenuListener() {
+            itemView.setOnLongClickListener(this);
+        }
+
+        public void onClick(View v){
+            openMediaFromView(this, v);
+        }
+
+        protected void openStorage() {
+            MediaWrapper mw = new MediaWrapper(((Storage) getItem(getAdapterPosition())).getUri());
+            mw.setType(MediaWrapper.TYPE_DIR);
+            fragment.browse(mw, getAdapterPosition(), binding.browserCheckbox.isChecked());
+        }
+
+        public void onCheckBoxClick(View v){
+            if (getItem(getAdapterPosition()) instanceof Storage) {
+                checkBoxAction(v, ((Storage) getItem(getAdapterPosition())).getUri().getPath());
+            }
+        }
+
+        public void onMoreClick(View v) {
+            fragment.openContextMenu(getLayoutPosition());
+        }
+
+        @Override
+        public boolean onLongClick(View v) {
+            fragment.mRecyclerView.openContextMenu(getLayoutPosition());
+            return true;
+        }
     }
 
-    public static class SeparatorViewHolder extends RecyclerView.ViewHolder {
+    public class SeparatorViewHolder extends ViewHolder {
         BrowserItemSeparatorBinding binding;
 
         public SeparatorViewHolder(View v) {
@@ -186,24 +223,36 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
         return mMediaList.isEmpty();
     }
 
-    public void addItem(Media media, boolean notify, boolean top){
-        MediaWrapper mediaWrapper = new MediaWrapper(media);
-        addItem(mediaWrapper, notify, top);
-
+    public void addItem(Object item, boolean notify, boolean top){
+        addItem(item, notify, top, -1);
     }
 
-    public void addItem(Object item, boolean notify, boolean top){
-        int position = top ? 0 : mMediaList.size();
+    public void addItem(Object item, boolean notify, int position){
+        addItem(item, notify, false, position);
+    }
+
+    public void addItem(Object item, boolean notify, boolean top, int positionTo){
+        int position;
+        if (positionTo != -1)
+            position = positionTo;
+        else
+            position = top ? mTop : mMediaList.size();
+
+        if (item instanceof Media)
+            item = new MediaWrapper((Media) item);
+
         if (item instanceof MediaWrapper && ((MediaWrapper)item).getTitle().startsWith("."))
             return;
-        else if (item instanceof Media)
-            item = new MediaWrapper((Media) item);
 
         mMediaList.add(position, item);
         if (notify)
             notifyItemInserted(position);
     }
 
+    public void setTop (int top) {
+        mTop = top;
+    }
+
     public void setDescription(int position, String description){
         Object item = getItem(position);
         if (item instanceof MediaWrapper)
@@ -242,6 +291,21 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
         }
     }
 
+    public void removeItem(String path, boolean notify){
+        int position = -1;
+        for (int i = 0; i< getItemCount(); ++i) {
+            Object item = mMediaList.get(i);
+            if (item instanceof MediaWrapper && TextUtils.equals(path, ((MediaWrapper) item).getUri().toString()))
+                position = i;
+        }
+        if (position == -1)
+            return;
+        mMediaList.remove(position);
+        if (notify) {
+            notifyItemRemoved(position);
+        }
+    }
+
     public Object getItem(int position){
         return mMediaList.get(position);
     }
@@ -294,32 +358,23 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
                 return R.drawable.ic_browser_unknown_normal;
         }
     }
-
-    public ClickHandler mClickHandler = new ClickHandler();
-    public class ClickHandler {
-        public void onClick(View v){
-            openMediaFromView(v);
-        }
-
-        public void onCheckBoxClick(View v){
-            checkBoxAction(v);
-        }
-
-        public void onMoreClick(View v){
-            fragment.openContextMenu(((Integer) v.getTag()).intValue());
-        }
+    protected String getProtocol(MediaWrapper media) {
+        if (!fragment.isRootDirectory() || !(fragment instanceof NetworkBrowserFragment))
+            return null;
+        if (media.getType() != MediaWrapper.TYPE_DIR)
+            return null;
+        return media.getUri().getScheme();
     }
 
-    protected void openMediaFromView(View v) {
-        final MediaViewHolder holder = (MediaViewHolder) v.getTag(R.id.layout_item);
+    protected void checkBoxAction(View v, String path){}
+
+    protected void openMediaFromView(MediaViewHolder holder, View v) {
         final MediaWrapper mw = (MediaWrapper) getItem(holder.getAdapterPosition());
         mw.removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
 
         if (mw.getType() == MediaWrapper.TYPE_DIR)
             fragment.browse(mw, holder.getAdapterPosition(), true);
-        else if (mw.getType() == MediaWrapper.TYPE_VIDEO)
-            MediaUtils.openMedia(v.getContext(), mw);
-        else  if (mw.getType() == MediaWrapper.TYPE_AUDIO) {
+        else  if (mw.getType() == MediaWrapper.TYPE_AUDIO || mw.getType() == MediaWrapper.TYPE_VIDEO) {
             int position = 0;
             LinkedList<MediaWrapper> mediaLocations = new LinkedList<MediaWrapper>();
             MediaWrapper mediaItem;
@@ -332,18 +387,9 @@ public class BaseBrowserAdapter extends  RecyclerView.Adapter<RecyclerView.ViewH
                             position = mediaLocations.size() - 1;
                     }
                 }
-            MediaUtils.openList(v.getContext(), mediaLocations, position);
+            MediaUtils.openList(holder.itemView.getContext(), mediaLocations, position);
         } else {
-            MediaUtils.openStream(v.getContext(), mw.getLocation());
+            MediaUtils.openStream(holder.itemView.getContext(), mw.getLocation());
         }
     }
-
-    protected void checkBoxAction(View v){}
-
-    @Override
-    public boolean onLongClick(View v) {
-        final MediaViewHolder holder = (MediaViewHolder) v.getTag(R.id.layout_item);
-        fragment.mRecyclerView.openContextMenu(holder.getAdapterPosition());
-        return true;
-    }
 }
