@@ -28,7 +28,7 @@ import android.net.Uri;
 import android.os.Build;
 import android.os.Bundle;
 import android.os.Message;
-import android.support.design.widget.Snackbar;
+import android.support.design.widget.FloatingActionButton;
 import android.support.v4.app.Fragment;
 import android.support.v4.app.FragmentTransaction;
 import android.support.v7.widget.LinearLayoutManager;
@@ -50,11 +50,12 @@ import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.SecondaryActivity;
-import org.videolan.vlc.gui.video.VideoPlayerActivity;
+import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.view.ContextMenuRecyclerView;
 import org.videolan.vlc.gui.view.DividerItemDecoration;
 import org.videolan.vlc.gui.view.SwipeRefreshLayout;
 import org.videolan.vlc.interfaces.IRefreshable;
+import org.videolan.vlc.media.MediaDatabase;
 import org.videolan.vlc.media.MediaUtils;
 import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.FileUtils;
@@ -71,8 +72,11 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
     public static final String KEY_MRL = "key_mrl";
     public static final String KEY_MEDIA = "key_media";
     public static final String KEY_MEDIA_LIST = "key_media_list";
+    public static final String KEY_CONTENT_LIST = "key_content_list";
     public static final String KEY_POSITION = "key_list";
 
+    protected FloatingActionButton mAddDirectoryFAB;
+
     protected BrowserFragmentHandler mHandler;
     protected MediaBrowser mMediaBrowser;
     protected ContextMenuRecyclerView mRecyclerView;
@@ -84,7 +88,7 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
     protected int mSavedPosition = -1, mFavorites = 0;
     public boolean mRoot;
 
-    private SparseArray<ArrayList<MediaWrapper>> mMediaLists = new SparseArray<ArrayList<MediaWrapper>>();
+    private SparseArray<ArrayList<MediaWrapper>> mFoldersContentLists;
     private ArrayList<MediaWrapper> mediaList;
     public int mCurrentParsedPosition = 0;
 
@@ -102,8 +106,12 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
 
         if (bundle == null)
             bundle = getArguments();
+        else
+            mFoldersContentLists = (SparseArray<ArrayList<MediaWrapper>>) VLCApplication.getData(KEY_CONTENT_LIST);
+        if (mFoldersContentLists == null)
+            mFoldersContentLists = new SparseArray<>();
         if (bundle != null){
-            mediaList = bundle.getParcelableArrayList(KEY_MEDIA_LIST);
+            mediaList = (ArrayList<MediaWrapper>) VLCApplication.getData(KEY_MEDIA_LIST);
             if (mediaList != null)
                 mAdapter.addAll(mediaList);
             mCurrentMedia = bundle.getParcelable(KEY_MEDIA);
@@ -139,12 +147,6 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         return v;
     }
 
-    @Override
-    public void onStart() {
-        super.onStart();
-        mMediaBrowser = new MediaBrowser(VLCInstance.get(), this);
-    }
-
     public void onStop(){
         super.onStop();
         releaseBrowser();
@@ -160,7 +162,8 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
     public void onSaveInstanceState(Bundle outState){
         outState.putString(KEY_MRL, mMrl);
         outState.putParcelable(KEY_MEDIA, mCurrentMedia);
-        outState.putParcelableArrayList(KEY_MEDIA_LIST, mediaList);
+        VLCApplication.storeData(KEY_MEDIA_LIST, mediaList);
+        VLCApplication.storeData(KEY_CONTENT_LIST, mFoldersContentLists);
         if (mRecyclerView != null) {
             outState.putInt(KEY_POSITION, mLayoutManager.findFirstCompletelyVisibleItemPosition());
         }
@@ -199,9 +202,9 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
         Fragment next = createFragment();
         Bundle args = new Bundle();
-        ArrayList<MediaWrapper> list = mMediaLists != null ? mMediaLists.get(position) : null;
+        ArrayList<MediaWrapper> list = mFoldersContentLists != null ? mFoldersContentLists.get(position) : null;
         if(list != null && !list.isEmpty())
-            args.putParcelableArrayList(KEY_MEDIA_LIST, list);
+            VLCApplication.storeData(KEY_MEDIA_LIST, list);
         args.putParcelable(KEY_MEDIA, media);
         next.setArguments(args);
         ft.replace(R.id.fragment_placeholder, next, media.getLocation());
@@ -212,16 +215,17 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
 
     @Override
     public void onMediaAdded(int index, Media media) {
+        boolean empty = mAdapter.isEmpty();
         mAdapter.addItem(media, mReadyToDisplay && mRoot, mRoot);
-        if (mReadyToDisplay)
+        if (empty && mReadyToDisplay)
             updateEmptyView();
-        if (mRoot)
+        if (mRoot && mSwipeRefreshLayout.isRefreshing())
             mHandler.sendEmptyMessage(BrowserFragmentHandler.MSG_HIDE_LOADING);
     }
 
     @Override
     public void onMediaRemoved(int index, Media media) {
-        mAdapter.removeItem(index, mReadyToDisplay);
+        mAdapter.removeItem(media.getUri().toString(), mReadyToDisplay);
     }
 
     @Override
@@ -257,13 +261,20 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
      */
     protected void updateEmptyView(){
         if (mAdapter.isEmpty()){
-            mEmptyView.setVisibility(View.VISIBLE);
-            mRecyclerView.setVisibility(View.GONE);
-            mSwipeRefreshLayout.setEnabled(false);
+            if (mSwipeRefreshLayout.isRefreshing()) {
+                mEmptyView.setText(R.string.loading);
+                mEmptyView.setVisibility(View.VISIBLE);
+                mRecyclerView.setVisibility(View.GONE);
+            } else {
+                mEmptyView.setText(R.string.directory_empty);
+                mEmptyView.setVisibility(View.VISIBLE);
+                mRecyclerView.setVisibility(View.GONE);
+                mSwipeRefreshLayout.setRefreshing(false);
+            }
         } else if (mEmptyView.getVisibility() == View.VISIBLE) {
             mEmptyView.setVisibility(View.GONE);
             mRecyclerView.setVisibility(View.VISIBLE);
-            mSwipeRefreshLayout.setEnabled(true);
+            mSwipeRefreshLayout.setRefreshing(false);
         }
     }
 
@@ -296,7 +307,9 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
 
     @Override
     public void refresh() {
+        mHandler.sendEmptyMessageDelayed(BrowserFragmentHandler.MSG_SHOW_LOADING, 300);
         mAdapter.clear();
+        mFoldersContentLists.clear();
         if (mMediaBrowser == null)
             mMediaBrowser = new MediaBrowser(VLCInstance.get(), this);
         else
@@ -305,24 +318,23 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         if (mRoot)
             browseRoot();
         else
-            mMediaBrowser.browse(mCurrentMedia != null ? mCurrentMedia.getUri() : Uri.parse(mMrl));
-        mHandler.sendEmptyMessageDelayed(BrowserFragmentHandler.MSG_SHOW_LOADING, 300);
+            mMediaBrowser.browse(mCurrentMedia != null ? mCurrentMedia.getUri() : Uri.parse(mMrl), true);
     }
 
-
-    View.OnClickListener mCancelDeleteMediaListener = new View.OnClickListener() {
-        @Override
-        public void onClick(View view) {
-            mHandler.removeMessages(BrowserFragmentHandler.MSG_DELETE_MEDIA);
-        }
-    };
-
+    private void deleteMedia(final MediaWrapper mw) {
+        VLCApplication.runBackground(new Runnable() {
+            @Override
+            public void run() {
+                FileUtils.deleteFile(mw.getUri().getPath());
+                MediaDatabase.getInstance().removeMedia(mw.getUri());
+            }
+        });
+    }
 
     protected static class BrowserFragmentHandler extends WeakHandler<BaseBrowserFragment> {
 
         public static final int MSG_SHOW_LOADING = 0;
         public static final int MSG_HIDE_LOADING = 1;
-        public static final int MSG_DELETE_MEDIA = 2;
         public static final int MSG_REFRESH = 3;
 
         public BrowserFragmentHandler(BaseBrowserFragment owner) {
@@ -334,18 +346,12 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
             switch (msg.what){
                 case MSG_SHOW_LOADING:
                     fragment.mSwipeRefreshLayout.setRefreshing(true);
+                    fragment.updateEmptyView();
                     break;
                 case MSG_HIDE_LOADING:
                     removeMessages(MSG_SHOW_LOADING);
                     fragment.mSwipeRefreshLayout.setRefreshing(false);
-                    break;
-                case MSG_DELETE_MEDIA:
-                    FileUtils.asyncRecursiveDelete((String) msg.obj, new FileUtils.Callback() {
-                        @Override
-                        public void onResult(boolean success) {
-                            obtainMessage(MSG_REFRESH).sendToTarget();
-                        }
-                    });
+                    fragment.updateEmptyView();
                     break;
                 case MSG_REFRESH:
                     if (getOwner() != null && !getOwner().isDetached())
@@ -377,16 +383,10 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
 
     protected void setContextMenu(MenuInflater inflater, Menu menu, int position) {
         MediaWrapper mw = (MediaWrapper) mAdapter.getItem(position);
+        int type = mw.getType();
         boolean canWrite = this instanceof FileBrowserFragment && FileUtils.canWrite(mw.getUri().getPath());
-        boolean isAudio = mw.getType() == MediaWrapper.TYPE_AUDIO;
-        boolean isVideo = mw.getType() == MediaWrapper.TYPE_VIDEO;
-        if (isAudio || isVideo) {
-            inflater.inflate(R.menu.directory_view_file, menu);
-            menu.findItem(R.id.directory_view_delete).setVisible(canWrite);
-            menu.findItem(R.id.directory_view_info).setVisible(isVideo);
-        } else if (mw.getType() == MediaWrapper.TYPE_DIR) {
-            boolean isEmpty = mMediaLists.get(position) == null || mMediaLists.get(position).isEmpty();
-            if (canWrite || !isEmpty) {
+        if (type == MediaWrapper.TYPE_DIR) {
+            boolean isEmpty = mFoldersContentLists.get(position) == null || mFoldersContentLists.get(position).isEmpty();
                 inflater.inflate(R.menu.directory_view_dir, menu);
 //                if (canWrite) {
 //                    boolean nomedia = new File(mw.getLocation() + "/.nomedia").exists();
@@ -398,7 +398,19 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
 //                }
                 menu.findItem(R.id.directory_view_play_folder).setVisible(!isEmpty);
                 menu.findItem(R.id.directory_view_delete).setVisible(canWrite);
+            if (this instanceof NetworkBrowserFragment) {
+                MediaDatabase db = MediaDatabase.getInstance();
+                if (db.networkFavExists(mw.getUri())) {
+                    menu.findItem(R.id.network_remove_favorite).setVisible(true);
+                    menu.findItem(R.id.network_edit_favorite).setVisible(true);
+                } else
+                    menu.findItem(R.id.network_add_favorite).setVisible(true);
             }
+        } else {
+            inflater.inflate(R.menu.directory_view_file, menu);
+            menu.findItem(R.id.directory_view_delete).setVisible(canWrite);
+            menu.findItem(R.id.directory_view_info).setVisible(type == MediaWrapper.TYPE_VIDEO || type == MediaWrapper.TYPE_AUDIO);
+            menu.findItem(R.id.directory_view_play_audio).setVisible(type != MediaWrapper.TYPE_AUDIO);
         }
     }
 
@@ -416,11 +428,11 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
         mRecyclerView.openContextMenu(position);
     }
 
-    protected boolean handleContextItemSelected(MenuItem item, int position) {
+    protected boolean handleContextItemSelected(MenuItem item, final int position) {
         int id = item.getItemId();
         if (! (mAdapter.getItem(position) instanceof MediaWrapper))
             return super.onContextItemSelected(item);
-        MediaWrapper mw = (MediaWrapper) mAdapter.getItem(position);
+        final MediaWrapper mw = (MediaWrapper) mAdapter.getItem(position);
         switch (id){
             case R.id.directory_view_play:
                 mw.removeFlags(MediaWrapper.MEDIA_FORCE_AUDIO);
@@ -432,17 +444,24 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
                 return true;
             }
             case R.id.directory_view_delete:
-                Snackbar.make(getView(), getString(R.string.file_deleted), Snackbar.LENGTH_LONG)
-                        .setAction(android.R.string.cancel, mCancelDeleteMediaListener)
-                        .show();
-                Message msg = mHandler.obtainMessage(BrowserFragmentHandler.MSG_DELETE_MEDIA, mw.getUri().getPath());
-                mHandler.sendMessageDelayed(msg, 3000);
+                mAdapter.removeItem(position, true);
+                UiTools.snackerWithCancel(getView(), getString(R.string.file_deleted), new Runnable() {
+                    @Override
+                    public void run() {
+                        deleteMedia(mw);
+                    }
+                }, new Runnable() {
+                    @Override
+                    public void run() {
+                        mAdapter.addItem(mw, true, position);
+                    }
+                });
                 return true;
             case  R.id.directory_view_info:
                 Intent i = new Intent(getActivity(), SecondaryActivity.class);
                 i.putExtra("fragment", "mediaInfo");
                 i.putExtra("param", mw.getUri().toString());
-                startActivity(i);
+                getActivity().startActivityForResult(i, MainActivity.ACTIVITY_RESULT_SECONDARY);
                 return true;
             case R.id.directory_view_play_audio: {
                 if (mService != null) {
@@ -451,12 +470,9 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
                 }
                 return true;
             }
-            case  R.id.directory_view_play_video:
-                VideoPlayerActivity.start(getActivity(), mw.getUri());
-                return true;
             case R.id.directory_view_play_folder:
                 ArrayList<MediaWrapper> mediaList = new ArrayList<MediaWrapper>();
-                for (MediaWrapper mediaItem : mMediaLists.get(position)){
+                for (MediaWrapper mediaItem : mFoldersContentLists.get(position)){
                     if (mediaItem.getType() == MediaWrapper.TYPE_AUDIO || mediaItem.getType() == MediaWrapper.TYPE_VIDEO)
                         mediaList.add(mediaItem);
                 }
@@ -472,14 +488,16 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
 //                if (new File(mw.getLocation()+"/.nomedia").delete())
 //                    updateLib();
 //                return true;
+
         }
         return false;
     }
 
     protected void parseSubDirectories() {
-        if (mCurrentParsedPosition == -1 || mAdapter.isEmpty())
+        if ((mRoot && this instanceof NetworkBrowserFragment) || mCurrentParsedPosition == -1 ||
+                mAdapter.isEmpty() || this instanceof FilePickerFragment)
             return;
-        mMediaLists.clear();
+        mFoldersContentLists.clear();
         if (mMediaBrowser == null)
             mMediaBrowser = new MediaBrowser(VLCInstance.get(), mFoldersBrowserListener);
         else
@@ -498,7 +516,7 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
                 mw = null;
             if (mw != null){
                 if (mw.getType() == MediaWrapper.TYPE_DIR || mw.getType() == MediaWrapper.TYPE_PLAYLIST){
-                    mMediaBrowser.browse(mw.getUri());
+                    mMediaBrowser.browse(mw.getUri(), false);
                     return;
                 }
             }
@@ -535,7 +553,7 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
             if (!TextUtils.equals(holderText, "")) {
                 mAdapter.setDescription(mCurrentParsedPosition, holderText);
                 directories.addAll(files);
-                mMediaLists.put(mCurrentParsedPosition, new ArrayList<MediaWrapper>(directories));
+                mFoldersContentLists.put(mCurrentParsedPosition, new ArrayList<MediaWrapper>(directories));
             }
             while (++mCurrentParsedPosition < mAdapter.getItemCount()){ //skip media that are not browsable
                 if (mAdapter.getItem(mCurrentParsedPosition) instanceof MediaWrapper) {
@@ -551,7 +569,7 @@ public abstract class BaseBrowserFragment extends MediaBrowserFragment implement
 
             if (mw != null) {
                 if (mCurrentParsedPosition < mAdapter.getItemCount()) {
-                    mMediaBrowser.browse(mw.getUri());
+                    mMediaBrowser.browse(mw.getUri(), false);
                 } else {
                     mCurrentParsedPosition = -1;
                     releaseBrowser();
