@@ -41,6 +41,8 @@ public class PreferencesFragment extends BasePreferenceFragment implements Share
 
     public final static String TAG = "VLC/PreferencesFragment";
 
+    public final static String PLAYBACK_HISTORY = "playback_history";
+
     @Override
     protected int getXml() {
         return R.xml.preferences;
@@ -55,11 +57,6 @@ public class PreferencesFragment extends BasePreferenceFragment implements Share
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
 
-        if (!AndroidDevices.hasTsp()){
-            findPreference("screen_orientation").setEnabled(false);
-            findPreference("enable_black_theme").setEnabled(false);
-        }
-
         // Screen orientation
         ListPreference screenOrientationPref = (ListPreference) findPreference("screen_orientation");
         screenOrientationPref.setOnPreferenceChangeListener(new Preference.OnPreferenceChangeListener() {
@@ -112,6 +109,9 @@ public class PreferencesFragment extends BasePreferenceFragment implements Share
             case "dev_category":
                 loadFragment(new Developer());
                 break;
+            case PLAYBACK_HISTORY:
+                getActivity().setResult(PreferencesActivity.RESULT_RESTART);
+                return true;
             default:
                 return super.onPreferenceTreeClick(preference);
         }
