@@ -27,11 +27,15 @@ import android.os.Bundle;
 import android.support.v7.preference.Preference;
 import android.support.v7.preference.TwoStatePreference;
 
+import org.videolan.libvlc.util.AndroidUtil;
 import org.videolan.vlc.PlaybackService;
 import org.videolan.vlc.R;
 import org.videolan.vlc.util.AndroidDevices;
 
 public class PreferencesUi extends BasePreferenceFragment {
+
+    public static final String KEY_ENABLE_TOUCH_PLAYER = "enable_touch_player";
+
     @Override
     protected int getXml() {
         return R.xml.preferences_ui;
@@ -46,13 +50,7 @@ public class PreferencesUi extends BasePreferenceFragment {
     public void onCreate(Bundle savedInstanceState) {
         super.onCreate(savedInstanceState);
 
-        if (!AndroidDevices.hasTsp()){
-            findPreference("enable_clone_mode").setEnabled(false);
-            findPreference("force_list_portrait").setEnabled(false);
-            findPreference("enable_brightness_gesture").setEnabled(false);
-            findPreference("enable_headset_detection").setEnabled(false);
-            findPreference("enable_steal_remote_control").setEnabled(false);
-        }
+        findPreference("tv_ui").setVisible(AndroidUtil.isJellyBeanMR1OrLater());
     }
 
     @Override
@@ -67,6 +65,7 @@ public class PreferencesUi extends BasePreferenceFragment {
                 PlaybackService.Client.restartService(getActivity());
                 return true;
             case "force_list_portrait":
+            case "tv_ui":
                 ((PreferencesActivity) getActivity()).setRestart();
                 return true;
         }
