@@ -45,8 +45,13 @@ import org.videolan.libvlc.util.Extensions;
 import org.videolan.libvlc.util.VLCUtil;
 import org.videolan.vlc.R;
 import org.videolan.vlc.VLCApplication;
+import org.videolan.vlc.gui.helpers.AudioUtil;
 import org.videolan.vlc.gui.helpers.BitmapUtil;
+import org.videolan.vlc.gui.helpers.UiTools;
+import org.videolan.vlc.gui.preferences.PreferencesActivity;
+import org.videolan.vlc.media.MediaDatabase;
 import org.videolan.vlc.media.MediaLibrary;
+import org.videolan.vlc.media.MediaUtils;
 import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.Strings;
@@ -55,8 +60,6 @@ import org.videolan.vlc.util.WeakHandler;
 
 import java.io.File;
 import java.nio.ByteBuffer;
-import java.util.concurrent.ExecutorService;
-import java.util.concurrent.Executors;
 
 public class MediaInfoFragment extends ListFragment {
 
@@ -80,7 +83,6 @@ public class MediaInfoFragment extends ListFragment {
     private final static int HIDE_DELETE = 3;
     private final static int EXIT = 4;
     private final static int SHOW_SUBTITLES = 5;
-    ExecutorService mThreadPoolExecutor;
 
     @Override
     public void onCreate(Bundle savedInstanceState) {
@@ -101,15 +103,14 @@ public class MediaInfoFragment extends ListFragment {
         mDelete = (ImageButton) v.findViewById(R.id.info_delete);
         mSubtitles = (ImageView) v.findViewById(R.id.info_subtitles);
 
-        mThreadPoolExecutor = Executors.newFixedThreadPool(2);
-        mThreadPoolExecutor.submit(mCheckFile);
-        mThreadPoolExecutor.submit(mLoadImage);
+        VLCApplication.runBackground(mCheckFile);
+        VLCApplication.runBackground(mLoadImage);
 
         mPathView.setText(mItem == null ? "" : Uri.decode(mItem.getLocation().substring(7)));
         mPlayButton.setOnClickListener(new OnClickListener() {
             @Override
             public void onClick(View v) {
-                VideoPlayerActivity.start(getActivity(), mItem.getUri());
+                MediaUtils.openMedia(getContext(), mItem);
             }
         });
 
@@ -117,15 +118,7 @@ public class MediaInfoFragment extends ListFragment {
             @Override
             public void onClick(View v) {
                 if (mItem != null) {
-                    VLCApplication.runBackground(new Runnable() {
-                        @Override
-                        public void run() {
-                            boolean deleted = FileUtils.deleteFile(mItem.getLocation());
-                            if (deleted) {
-                                mHandler.obtainMessage(EXIT).sendToTarget();
-                            }
-                        }
-                    });
+                    UiTools.snackerWithCancel(getView(), getString(R.string.file_deleted), mDeleteAction);
                 }
             }
         });
@@ -150,8 +143,8 @@ public class MediaInfoFragment extends ListFragment {
 
     public void onStop(){
         super.onStop();
-        if (mThreadPoolExecutor != null)
-            mThreadPoolExecutor.shutdownNow();
+        VLCApplication.removeTask(mCheckFile);
+        VLCApplication.removeTask(mLoadImage);
         if (mMedia != null)
             mMedia.release();
     }
@@ -162,6 +155,12 @@ public class MediaInfoFragment extends ListFragment {
         outState.putParcelable(ITEM_KEY, mItem);
     }
 
+    @Override
+    public void onDestroy() {
+        super.onDestroy();
+        mHandler = null;
+    }
+
     public void setMediaLocation(String MRL) {
         if (MRL == null)
             return;
@@ -176,7 +175,8 @@ public class MediaInfoFragment extends ListFragment {
                 mHandler.obtainMessage(HIDE_DELETE).sendToTarget();
             long length = itemFile.length();
             mHandler.obtainMessage(NEW_SIZE, Long.valueOf(length)).sendToTarget();
-            checkSubtitles(itemFile);
+            if (mItem.getType() == MediaWrapper.TYPE_VIDEO)
+                checkSubtitles(itemFile);
         }
     };
 
@@ -211,6 +211,11 @@ public class MediaInfoFragment extends ListFragment {
             extension = filename.substring(index);
             if (!Extensions.SUBTITLES.contains(extension))
                 continue;
+
+            if (mHandler == null || Thread.interrupted()) {
+                return;
+            }
+
             if (filename.startsWith(videoName)) {
                 mHandler.obtainMessage(SHOW_SUBTITLES).sendToTarget();
                 return;
@@ -224,8 +229,6 @@ public class MediaInfoFragment extends ListFragment {
             final LibVLC libVlc = VLCInstance.get();
             if (libVlc == null)
                 return;
-            mMedia = new Media(libVlc, mItem.getUri());
-            mMedia.parse();
             int videoHeight = mItem.getHeight();
             int videoWidth = mItem.getWidth();
             if (videoWidth == 0 || videoHeight == 0) {
@@ -234,6 +237,12 @@ public class MediaInfoFragment extends ListFragment {
                 videoHeight = 9;
             }
 
+            if (mHandler == null || Thread.interrupted()) {
+                return;
+            }
+            mMedia = new Media(libVlc, mItem.getUri());
+            mMedia.parse();
+
             mHandler.sendEmptyMessage(NEW_TEXT);
 
             DisplayMetrics screen = new DisplayMetrics();
@@ -246,19 +255,26 @@ public class MediaInfoFragment extends ListFragment {
             }
             height = width * videoHeight/videoWidth;
 
-            // Get the thumbnail.
-            mImage = Bitmap.createBitmap(width, height, Config.ARGB_8888);
+            if (mItem.getType() == MediaWrapper.TYPE_VIDEO) {
+                // Get the thumbnail.
+                mImage = Bitmap.createBitmap(width, height, Config.ARGB_8888);
 
-            byte[] b = VLCUtil.getThumbnail(mMedia, width, height);
+                byte[] b = VLCUtil.getThumbnail(mMedia, width, height);
 
-            if (b == null) // We were not able to create a thumbnail for this item.
+                if (b == null) // We were not able to create a thumbnail for this item.
+                    return;
+                mImage.copyPixelsFromBuffer(ByteBuffer.wrap(b));
+                mImage = BitmapUtil.cropBorders(mImage, width, height);
+            } else if (mItem.getType() == MediaWrapper.TYPE_AUDIO) {
+                mImage = AudioUtil.getCover(getActivity(), mItem, width);
+                if (mImage == null)
+                    return;
+            } else
                 return;
 
-            if (Thread.interrupted()) {
+            if (mHandler == null || Thread.interrupted()) {
                 return;
             }
-            mImage.copyPixelsFromBuffer(ByteBuffer.wrap(b));
-            mImage = BitmapUtil.cropBorders(mImage, width, height);
 
             mHandler.sendEmptyMessage(NEW_IMAGE);
         }
@@ -274,7 +290,6 @@ public class MediaInfoFragment extends ListFragment {
         lp.width = mImage.getWidth();
         imageView.setLayoutParams(lp);
         imageView.setScaleType(ImageView.ScaleType.CENTER_CROP);
-        mPlayButton.setVisibility(View.VISIBLE);
         mLengthView.setVisibility(View.VISIBLE);
     }
 
@@ -289,10 +304,7 @@ public class MediaInfoFragment extends ListFragment {
                 hasSubs = true;
             mAdapter.add(track);
         }
-        if (mAdapter.isEmpty()) {
-            getActivity().finish();
-            return;
-        }
+
         if (hasSubs)
             mHandler.obtainMessage(SHOW_SUBTITLES).sendToTarget();
     }
@@ -300,6 +312,21 @@ public class MediaInfoFragment extends ListFragment {
     private void updateSize(Long size){
         mSizeView.setText(Strings.readableFileSize(size.longValue()));
     }
+
+    private Runnable mDeleteAction = new Runnable() {
+        @Override
+        public void run() {
+            VLCApplication.runBackground(new Runnable() {
+                @Override
+                public void run() {
+                    FileUtils.deleteFile(mItem.getUri().getPath());
+                    MediaDatabase.getInstance().removeMedia(mItem.getUri());
+                    mHandler.sendEmptyMessage(EXIT);
+                }
+            });
+        }
+    };
+
     private Handler mHandler = new MediaInfoHandler(this);
 
     private static class MediaInfoHandler extends WeakHandler<MediaInfoFragment> {
@@ -309,7 +336,7 @@ public class MediaInfoFragment extends ListFragment {
 
         @Override
         public void handleMessage(Message msg) {
-            MediaInfoFragment fragment = getOwner();
+            final MediaInfoFragment fragment = getOwner();
             if(fragment == null) return;
 
             switch (msg.what) {
@@ -327,8 +354,8 @@ public class MediaInfoFragment extends ListFragment {
                     fragment.mDelete.setVisibility(View.GONE);
                     break;
                 case EXIT:
+                    fragment.getActivity().setResult(PreferencesActivity.RESULT_RESCAN);
                     fragment.getActivity().finish();
-                    MediaLibrary.getInstance().scanMediaItems(true);
                     break;
                 case SHOW_SUBTITLES:
                     fragment.mSubtitles.setVisibility(View.VISIBLE);
