@@ -29,9 +29,7 @@ import android.content.res.Configuration;
 import android.content.res.Resources;
 import android.os.Bundle;
 import android.os.Handler;
-import android.os.Message;
 import android.preference.PreferenceManager;
-import android.support.design.widget.Snackbar;
 import android.support.v4.app.FragmentActivity;
 import android.support.v4.content.LocalBroadcastManager;
 import android.support.v4.util.ArrayMap;
@@ -54,6 +52,7 @@ import org.videolan.vlc.VLCApplication;
 import org.videolan.vlc.gui.MainActivity;
 import org.videolan.vlc.gui.SecondaryActivity;
 import org.videolan.vlc.gui.browser.MediaBrowserFragment;
+import org.videolan.vlc.gui.helpers.UiTools;
 import org.videolan.vlc.gui.view.ContextMenuRecyclerView;
 import org.videolan.vlc.gui.view.SwipeRefreshLayout;
 import org.videolan.vlc.interfaces.ISortable;
@@ -66,7 +65,6 @@ import org.videolan.vlc.media.MediaWrapper;
 import org.videolan.vlc.media.Thumbnailer;
 import org.videolan.vlc.util.FileUtils;
 import org.videolan.vlc.util.VLCInstance;
-import org.videolan.vlc.util.WeakHandler;
 import org.videolan.vlc.view.AutoFitRecyclerView;
 
 import java.util.ArrayList;
@@ -78,9 +76,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
     public final static String KEY_GROUP = "key_group";
 
-    private static final int DELETE_MEDIA = 0;
-    private static final int DELETE_DURATION = 3000;
-
     protected LinearLayout mLayoutFlipperLoading;
     protected AutoFitRecyclerView mGridView;
     protected TextView mTextViewNomedia;
@@ -127,7 +122,6 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
 
         mGridView.addOnScrollListener(mScrollListener);
         mGridView.setAdapter(mVideoAdapter);
-        mGridView.setItemAnimator(null);
         return v;
     }
 
@@ -270,10 +264,10 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         }
     }
 
-    private boolean handleContextItemSelected(MenuItem menu, int position) {
+    private boolean handleContextItemSelected(MenuItem menu, final int position) {
         if (position >= mVideoAdapter.getItemCount())
             return false;
-        MediaWrapper media = mVideoAdapter.getItem(position);
+        final MediaWrapper media = mVideoAdapter.getItem(position);
         if (media == null)
             return false;
         switch (menu.getItemId()){
@@ -283,6 +277,23 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
             case R.id.video_list_play_audio:
                 playAudio(media);
                 return true;
+            case R.id.video_list_play_all:
+                ArrayList<MediaWrapper> playList = new ArrayList<>();
+                ArrayList<MediaWrapper> videos = mVideoAdapter.getAll();
+                MediaWrapper mw;
+                int offset = 0;
+                for (int i = 0; i < videos.size(); ++i) {
+                    mw = videos.get(i);
+                    if (mw instanceof MediaGroup) {
+                        for (MediaWrapper item : ((MediaGroup) mw).getAll())
+                            playList.add(item);
+                        if (i < position)
+                            offset += ((MediaGroup)mw).size()-1;
+                    } else
+                        playList.add(mw);
+                }
+                MediaUtils.openList(getActivity(), playList, position+offset);
+                return true;
             case R.id.video_list_info:
                 Activity activity = getActivity();
                 if (activity instanceof MainActivity)
@@ -291,16 +302,30 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
                     Intent i = new Intent(activity, SecondaryActivity.class);
                     i.putExtra("fragment", "mediaInfo");
                     i.putExtra("param", media.getLocation());
-                    startActivity(i);
+                    startActivityForResult(i, SecondaryActivity.ACTIVITY_RESULT_SECONDARY);
                 }
                 return true;
             case R.id.video_list_delete:
-                Snackbar.make(getView(), getString(R.string.file_deleted), DELETE_DURATION)
-                    .setAction(android.R.string.cancel, mCancelDeleteMediaListener)
-                    .show();
-                Message msg = mDeleteHandler.obtainMessage(DELETE_MEDIA, position, 0);
-                mDeleteHandler.sendMessageDelayed(msg, DELETE_DURATION);
+                mVideoAdapter.remove(position);
+                UiTools.snackerWithCancel(getView(), getString(R.string.file_deleted), new Runnable() {
+                    @Override
+                    public void run() {
+                        deleteMedia(media);
+                    }
+                }, new Runnable() {
+                    @Override
+                    public void run() {
+                        mVideoAdapter.add(position, media);
+                    }
+                });
                 return true;
+            case R.id.video_group_play:
+                MediaUtils.openList(getActivity(), ((MediaGroup) media).getAll(), 0);
+            case R.id.video_list_append:
+                if (media instanceof MediaGroup)
+                    mService.append(((MediaGroup)media).getAll());
+                else
+                    mService.append(media);
         }
         return false;
     }
@@ -312,11 +337,12 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         // Do not show the menu of media group.
         ContextMenuRecyclerView.RecyclerContextMenuInfo info = (ContextMenuRecyclerView.RecyclerContextMenuInfo)menuInfo;
         MediaWrapper media = mVideoAdapter.getItem(info.position);
-        if (media == null || media instanceof MediaGroup)
+        if (media == null)
             return;
         MenuInflater inflater = getActivity().getMenuInflater();
-        inflater.inflate(R.menu.video_list, menu);
-        setContextMenuItems(menu, media);
+        inflater.inflate(media instanceof MediaGroup ? R.menu.video_group_contextual : R.menu.video_list, menu);
+        if (!(media instanceof MediaGroup))
+            setContextMenuItems(menu, media);
     }
 
     private void setContextMenuItems(Menu menu, MediaWrapper mediaWrapper) {
@@ -360,7 +386,7 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         activity.setMenuFocusDown(idIsEmpty, android.R.id.list);
         activity.setSearchAsFocusDown(idIsEmpty, parent,
                 android.R.id.list);
-        }
+    }
 
     public void updateList() {
         if (!mSwipeRefreshLayout.isRefreshing())
@@ -493,14 +519,15 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
         mVideoAdapter.clear();
     }
 
-    public void deleteMedia(int position){
-        final MediaWrapper media = mVideoAdapter.getItem(position);
-        if (media == null)
-            return;
-        final String path = media.getUri().getPath();
-        FileUtils.asyncRecursiveDelete(path);
+    public void deleteMedia(final MediaWrapper media){
+        VLCApplication.runBackground(new Runnable() {
+            @Override
+            public void run() {
+                FileUtils.deleteFile(media.getUri().getPath());
+                MediaDatabase.getInstance().removeMedia(media.getUri());
+            }
+        });
         mMediaLibrary.getMediaItems().remove(media);
-        mVideoAdapter.remove(media);
         if (mService != null) {
             final List<String> list = mService.getMediaLocations();
             if (list != null && list.contains(media.getLocation())) {
@@ -508,30 +535,4 @@ public class VideoGridFragment extends MediaBrowserFragment implements ISortable
             }
         }
     }
-
-
-    View.OnClickListener mCancelDeleteMediaListener = new View.OnClickListener() {
-        @Override
-        public void onClick(View view) {
-            mDeleteHandler.removeMessages(DELETE_MEDIA);
-        }
-    };
-
-    Handler mDeleteHandler = new VideoDeleteHandler(this);
-
-    private static class VideoDeleteHandler extends WeakHandler<VideoGridFragment>{
-
-        public VideoDeleteHandler(VideoGridFragment owner) {
-            super(owner);
-        }
-
-        @Override
-        public void handleMessage(Message msg) {
-            super.handleMessage(msg);
-            switch (msg.what){
-                case DELETE_MEDIA:
-                    getOwner().deleteMedia(msg.arg1);
-            }
-        }
-    }
 }
