@@ -53,7 +53,7 @@ import java.util.Comparator;
 import java.util.Locale;
 
 public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.ViewHolder>
-        implements Comparator<MediaWrapper>, View.OnLongClickListener {
+        implements Comparator<MediaWrapper> {
 
     public final static String TAG = "VLC/VideoListAdapter";
 
@@ -105,11 +105,8 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
         fillView(holder, media);
 
-        holder.binding.setVariable(BR.position, position);
         holder.binding.setVariable(BR.media, media);
-        holder.binding.setVariable(BR.handler, mClickHandler);
         holder.binding.executePendingBindings();
-        holder.itemView.setOnLongClickListener(this);
         if (asyncLoad)
             AsyncImageLoader.LoadImage(new VideoCoverFetcher(holder.binding, media), null);
     }
@@ -213,15 +210,22 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
         mVideos.add(item);
     }
 
+    public void add(int position, MediaWrapper item) {
+        mVideos.add(position, item);
+        notifyItemInserted(position);
+    }
+
     @MainThread
     public void remove(MediaWrapper item) {
-        int position = getItemPosition(item);
+        remove(getItemPosition(item));
+    }
+
+    @MainThread
+    public void remove(int position) {
         if (position == -1)
             return;
         mVideos.remove(position);
-        // Remove the whole end of list to update position tags stored in
-        // items views for databinder interactions, like contextual menu
-        notifyItemRangeChanged(position, getItemCount()-position);
+        notifyItemRemoved(position);
     }
 
     private int getItemPosition(MediaWrapper mw) {
@@ -240,6 +244,10 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
 
     }
 
+    public ArrayList<MediaWrapper> getAll() {
+        return mVideos;
+    }
+
     @MainThread
     public void update(MediaWrapper item) {
         int position = mVideos.indexOf(item);
@@ -254,19 +262,16 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
     }
 
     private void fillView(ViewHolder holder, MediaWrapper media) {
-        boolean group;
         String text = "";
         String resolution = "";
         int max = 0;
         int progress = 0;
 
         if (media.getType() == MediaWrapper.TYPE_GROUP) {
-            group = true;
             MediaGroup mediaGroup = (MediaGroup) media;
             int size = mediaGroup.size();
             resolution = VLCApplication.getAppResources().getQuantityString(R.plurals.videos_quantity, size, size);
         } else {
-            group = false;
             /* Time / Duration */
             if (media.getLength() > 0) {
                 long lastTime = media.getTime();
@@ -284,7 +289,6 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
                 resolution = String.format("%dx%d", media.getWidth(), media.getHeight());
         }
 
-        holder.binding.setVariable(BR.group, group);
         holder.binding.setVariable(BR.resolution, resolution);
         holder.binding.setVariable(BR.time, text);
         holder.binding.setVariable(BR.max, max);
@@ -336,7 +340,7 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
         return super.getItemViewType(position);
     }
 
-    static class ViewHolder extends RecyclerView.ViewHolder {
+    public class ViewHolder extends RecyclerView.ViewHolder implements View.OnLongClickListener {
         boolean listmode;
         ViewDataBinding binding;
 
@@ -344,13 +348,12 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
             super(itemView);
             binding = DataBindingUtil.bind(itemView);
             this.listmode = listMode;
+            binding.setVariable(BR.holder, this);
+            itemView.setOnLongClickListener(this);
         }
-    }
 
-    public ClickHandler mClickHandler = new ClickHandler();
-    public class ClickHandler {
         public void onClick(View v){
-            MediaWrapper media = mVideos.get(((Integer) v.findViewById(R.id.item_more).getTag()).intValue());
+            MediaWrapper media = mVideos.get(getAdapterPosition());
             if (media instanceof MediaGroup) {
                 MainActivity activity = (MainActivity) mFragment.getActivity();
                 activity.showSecondaryFragment(SecondaryActivity.VIDEO_GROUP_LIST, media.getTitle());
@@ -359,19 +362,19 @@ public class VideoListAdapter extends RecyclerView.Adapter<VideoListAdapter.View
                 VideoPlayerActivity.start(v.getContext(), media.getUri(), media.getTitle());
             }
         }
+
         public void onMoreClick(View v){
             if (mFragment == null)
                 return;
-            mFragment.mGridView.openContextMenu(((Integer) v.getTag()).intValue());
+            mFragment.mGridView.openContextMenu(getAdapterPosition());
+        }
+
+        @Override
+        public boolean onLongClick(View v) {
+            if (mFragment == null)
+                return false;
+            mFragment.mGridView.openContextMenu(getLayoutPosition());
+            return true;
         }
     }
-
-    @Override
-    public boolean onLongClick(View v) {
-        if (mFragment == null)
-            return false;
-        int position = ((Integer) v.findViewById(R.id.item_more).getTag()).intValue();
-        mFragment.mGridView.openContextMenu(position);
-        return true;
-    }
 }
