@@ -32,19 +32,41 @@ public class MediaUtils {
         LocalBroadcastManager.getInstance(VLCApplication.getAppContext()).sendBroadcast(intent);
     }
 
+    public static void appendMedia(final Context context, final MediaWrapper media){
+        if (media == null)
+            return;
+        new DialogCallback(context, new DialogCallback.Runnable() {
+                @Override
+                public void run(PlaybackService service) {
+                    service.append(media);
+                }
+        });
+    }
+
     public static void openMedia(final Context context, final MediaWrapper media){
         if (media == null)
             return;
-        if (media.getType() == MediaWrapper.TYPE_VIDEO)
-            VideoPlayerActivity.start(context, media.getUri(), media.getTitle());
-        else if (media.getType() == MediaWrapper.TYPE_AUDIO) {
-            new DialogCallback(context, new DialogCallback.Runnable() {
+        new DialogCallback(context, new DialogCallback.Runnable() {
                 @Override
                 public void run(PlaybackService service) {
                     service.load(media);
                 }
-            });
-        }
+        });
+    }
+
+    public static void openMediaNoUi(final Context context, final MediaWrapper media){
+        if (media == null)
+            return;
+        if (media.getType() == MediaWrapper.TYPE_VIDEO)
+            VideoPlayerActivity.start(context, media.getUri(), media.getTitle());
+        else
+            new BaseCallBack(context) {
+                @Override
+                public void onConnected(PlaybackService service) {
+                    service.load(media);
+                    mClient.disconnect();
+                }
+            };
     }
 
     public static void openList(final Context context, final List<MediaWrapper> list, final int position){
@@ -133,9 +155,22 @@ public class MediaUtils {
         }
     }
 
-    private static class DialogCallback implements PlaybackService.Client.Callback {
+    private static abstract class BaseCallBack implements PlaybackService.Client.Callback {
+        protected PlaybackService.Client mClient;
+
+        private BaseCallBack(Context context) {
+            mClient = new PlaybackService.Client(context, this);
+            mClient.connect();
+        }
+
+        protected BaseCallBack() {}
+
+        @Override
+        public void onDisconnected() {}
+    }
+
+    private static class DialogCallback extends BaseCallBack {
         private final ProgressDialog dialog;
-        final private PlaybackService.Client mClient;
         final private Runnable mRunnable;
 
         private interface Runnable {
@@ -162,7 +197,7 @@ public class MediaUtils {
         @Override
         public void onConnected(PlaybackService service) {
             mRunnable.run(service);
-            dialog.dismiss();
+            dialog.cancel();
         }
 
         @Override
