@@ -21,26 +21,34 @@
 package org.videolan.vlc.util;
 
 import android.annotation.TargetApi;
+import android.content.ComponentName;
 import android.content.Context;
+import android.content.pm.PackageManager;
 import android.net.ConnectivityManager;
 import android.net.NetworkInfo;
 import android.os.Build.VERSION;
 import android.os.Build.VERSION_CODES;
 import android.os.Environment;
 import android.telephony.TelephonyManager;
+import android.text.TextUtils;
 import android.view.InputDevice;
 import android.view.MotionEvent;
 
 import org.videolan.libvlc.util.AndroidUtil;
+import org.videolan.vlc.R;
+import org.videolan.vlc.RemoteControlClientReceiver;
 import org.videolan.vlc.VLCApplication;
+import org.videolan.vlc.media.MediaWrapper;
 
 import java.io.BufferedReader;
+import java.io.File;
 import java.io.FileNotFoundException;
 import java.io.FileReader;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashSet;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.StringTokenizer;
 
@@ -49,7 +57,7 @@ public class AndroidDevices {
     public final static String EXTERNAL_PUBLIC_DIRECTORY = Environment.getExternalStorageDirectory().getPath();
 
     final static boolean hasNavBar;
-    final static boolean hasTsp;
+    final static boolean hasTsp, isTv;
 
     static {
         HashSet<String> devicesWithoutNavBar = new HashSet<String>();
@@ -60,6 +68,7 @@ public class AndroidDevices {
         hasNavBar = AndroidUtil.isICSOrLater()
                 && !devicesWithoutNavBar.contains(android.os.Build.MODEL);
         hasTsp = VLCApplication.getAppContext().getPackageManager().hasSystemFeature("android.hardware.touchscreen");
+        isTv = VLCApplication.getAppContext().getPackageManager().hasSystemFeature("android.software.leanback");
     }
 
     public static boolean hasExternalStorage() {
@@ -88,6 +97,10 @@ public class AndroidDevices {
         return hasTsp;
     }
 
+    public static boolean isAndroidTv() {
+        return isTv;
+    }
+
     public static ArrayList<String> getStorageDirectories() {
         BufferedReader bufReader = null;
         ArrayList<String> list = new ArrayList<String>();
@@ -139,6 +152,22 @@ public class AndroidDevices {
         return list;
     }
 
+    public static List<MediaWrapper> getMediaDirectoriesList() {
+        String storages[] = AndroidDevices.getMediaDirectories();
+        LinkedList<MediaWrapper> list = new LinkedList<>();
+        MediaWrapper directory;
+        for (String mediaDirLocation : storages) {
+            if (!(new File(mediaDirLocation).exists()))
+                continue;
+            directory = new MediaWrapper(AndroidUtil.PathToUri(mediaDirLocation));
+            directory.setType(MediaWrapper.TYPE_DIR);
+            if (TextUtils.equals(AndroidDevices.EXTERNAL_PUBLIC_DIRECTORY, mediaDirLocation))
+                directory.setDisplayTitle(VLCApplication.getAppResources().getString(R.string.internal_memory));
+            list.add(directory);
+        }
+        return list;
+    }
+
     public static String[] getMediaDirectories() {
         ArrayList<String> list = new ArrayList<String>();
         list.addAll(getStorageDirectories());
@@ -168,6 +197,14 @@ public class AndroidDevices {
         return 0;
     }
 
+    public static boolean hasPlayServices() {
+        try {
+            VLCApplication.getAppContext().getPackageManager().getPackageInfo("com.google.android.gsf", PackageManager.GET_SERVICES);
+            return true;
+        } catch (PackageManager.NameNotFoundException e) {}
+        return false;
+    }
+
     public static boolean hasLANConnection() {
         boolean networkEnabled = false;
         ConnectivityManager connectivity = (ConnectivityManager) (VLCApplication.getAppContext().getSystemService(Context.CONNECTIVITY_SERVICE));
@@ -180,4 +217,37 @@ public class AndroidDevices {
         }
         return networkEnabled;
     }
+
+    public static boolean hasConnection() {
+        boolean networkEnabled = false;
+        ConnectivityManager connectivity = (ConnectivityManager) (VLCApplication.getAppContext().getSystemService(Context.CONNECTIVITY_SERVICE));
+        if (connectivity != null) {
+            NetworkInfo networkInfo = connectivity.getActiveNetworkInfo();
+            if (networkInfo != null && networkInfo.isConnected()) {
+                networkEnabled = true;
+            }
+        }
+        return networkEnabled;
+    }
+
+    public static boolean hasMobileConnection() {
+        boolean networkEnabled = false;
+        ConnectivityManager connectivity = (ConnectivityManager) (VLCApplication.getAppContext().getSystemService(Context.CONNECTIVITY_SERVICE));
+        if (connectivity != null) {
+            NetworkInfo networkInfo = connectivity.getActiveNetworkInfo();
+            if (networkInfo != null && networkInfo.isConnected() &&
+                    (networkInfo.getType() == ConnectivityManager.TYPE_MOBILE)) {
+                networkEnabled = true;
+            }
+        }
+        return networkEnabled;
+    }
+
+    public static void setRemoteControlReceiverEnabled(boolean enabled) {
+        VLCApplication.getAppContext().getPackageManager().setComponentEnabledSetting(
+                new ComponentName(VLCApplication.getAppContext(), RemoteControlClientReceiver.class),
+                enabled ? PackageManager.COMPONENT_ENABLED_STATE_ENABLED :
+                        PackageManager.COMPONENT_ENABLED_STATE_DISABLED,
+                PackageManager.DONT_KILL_APP);
+    }
 }
